package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantViewOutput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageInput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageOutput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageViewOutput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.entity.BaseResult;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.enums.BusinessError;
import pps.core.space.mapper.*;
import pps.core.space.service.data.*;
import pps.core.space.service.data.space_institution_detail.OptimizeWellResult;
import pps.core.space.service.data.space_optimize_effect.GetSpaceOptimizeEffectOutput;
import pps.core.space.service.data.space_optimize_period.GetPredictedPowerOutput;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizePeriodInput;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizePeriodOutput;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizeWellheadOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.error.XError;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 间开优化公用方法
 *
 * @author ZWT
 * @date 2023/09/18 16:02
 */
public class SpaceOptimizeBaseService {

    /**
     * 动态优化调接口 api
     *
     * @param context                上下文
     * @param periodDTOList          周期数据列表
     * @param wellheadDTOList        井口数据表
     * @param durationDTOList        持续时间数据列表
     * @param unOptimizeDurationList 未优化持续时间列表
     * @param dateType               日期类型
     * @param flag                   旗帜
     * @param offset                 抵消
     */
    public void dynamicOptimize4Api(XContext context, List<SpaceOptimizePeriodDTO> periodDTOList, List<SpaceOptimizeWellheadDTO> wellheadDTOList,
                                    List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                    Integer dateType, Integer flag, Integer offset) {
        //取当前时间
        DateTime startDate = DateUtil.beginOfDay(DateTime.now());
        //取生效中的基础间开
        List<SpaceInstitutionDetailEnt> detailEntList = this.getEffectiveSpaceInstitution(context, startDate);
        //取线路ID和制度ID
        List<String> lineIds = new ArrayList<>(32);
        List<String> institutionIds = new ArrayList<>(32);
        detailEntList.forEach(detail -> {
            lineIds.add(detail.getLineId());
            institutionIds.add(detail.getId());
        });
        //取间开的所有井口
        List<SpaceInstitutionWellheadView> spaceWellheadList = this.getSpaceWellheadList(context, institutionIds);
        if (CollUtil.isEmpty(spaceWellheadList)) {
            throw new XServiceException(BusinessError.DidNotFindWellhead);
        }
        //取间开对应的线路
        List<DynamicQueryBasePowerLineOutput> powerLineList = ServiceUtil.getPowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .lineIds(lineIds)
                .build());
        //查线路下所有电站
        List<DynamicQueryBasePowerLinePlantViewOutput> powerLinePlantList = ServiceUtil.getPowerLinePlantList(context, DynamicQueryBasePowerLinePlantInput.builder()
                .lineIds(lineIds)
                .build());
        //取当前月
        int month = startDate.monthBaseOne();
        //查井口配置时间段
        SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
        List<SpaceInstitutionDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDurationEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .in(SpaceInstitutionDurationEnt::getWellheadId, spaceWellheadList.stream().map(SpaceInstitutionWellheadView::getWellheadId).collect(Collectors.toList()))
                .orderByAsc(SpaceInstitutionDurationEnt::getOpenWellTime)
        );
        if (CollUtil.isEmpty(durationList)) {
            throw new XServiceException(BusinessError.DidNotFindCalibration);
        } else {
            for (SpaceInstitutionDurationEnt ent : durationList) {
                ent.setOpenTime(BaseUtils.strToDateTime(ent.getOpenWellTime()));
                ent.setCloseTime(BaseUtils.strToDateTime(ent.getCloseWellTime()));
            }
        }
        //间开制度ID/井口LIST Map
        Map<String, List<SpaceInstitutionWellheadView>> wellheadMap = spaceWellheadList.stream()
                .collect(Collectors.groupingBy(SpaceInstitutionWellheadView::getInstitutionId));
        //线路ID/峰谷策略ID Map
        Map<String, String> lineMap = powerLineList.stream()
                .collect(Collectors.toMap(DynamicQueryBasePowerLineOutput::getId, DynamicQueryBasePowerLineOutput::getStrategyId));
        //线路ID/电站ID LIST Map
        Map<String, List<String>> plantIdsMap = powerLinePlantList.stream()
                .collect(Collectors.groupingBy(DynamicQueryBasePowerLinePlantViewOutput::getLineId,
                        Collectors.mapping(DynamicQueryBasePowerLinePlantViewOutput::getPlantId, Collectors.toList())));
        //峰谷策略ID/峰谷策略LIST Map
        //市电峰谷
        Map<String, Map<String, List<GetBasePriceStrategyDetailOutput>>> strategyMap;
        List<GetBasePriceStrategyDetailOutput> strategyList = ServiceUtil.getStrategyDetailList(context, GetBasePriceStrategyDetailInput.builder()
                .monthList(CollUtil.list(false, String.valueOf(month), String.valueOf(month + 1)))
                .build());
        if (CollUtil.isNotEmpty(strategyList)) {
            strategyMap = strategyList.stream()
                    .collect(Collectors.groupingBy(GetBasePriceStrategyDetailOutput::getStrategyMonth,
                            Collectors.groupingBy(GetBasePriceStrategyDetailOutput::getStrategyId)));
        } else {
            strategyMap = Collections.emptyMap();
        }
        //间开制度ID/区间配置Map Map
        Map<String, Map<String, List<SpaceInstitutionDurationEnt>>> durationMap = durationList.stream()
                .collect(Collectors.groupingBy(SpaceInstitutionDurationEnt::getInstitutionId,
                        Collectors.groupingBy(SpaceInstitutionDurationEnt::getWellheadId)));
        //初始化时间轴
        List<DateTime> rangeToList = DateUtil.rangeToList(DateUtil.offsetDay(BusinessConstant.DATE_FLAG, -1), BusinessConstant.DATE_FLAG, DateField.MINUTE, 30);
        //遍历
        List<DynamicQueryPlantPredictedPowerOutput> powerList;
        List<SpaceInstitutionWellheadView> wellheadList;
        List<SpaceInstitutionWellheadView> unOptimizeWellhead;
        List<String> plantIds;
        String periodId;
        Map<Boolean, List<SpaceInstitutionWellheadView>> collect;
        //时间间隔
        Integer startInterval;
        Date institutionEndDate;
        DateTime optimizeDeadline;
        //取优化截至日期
        DateTime optimizeDate;
        DateTime begin;
        DateTime recordTime;
        String monthBaseOne;
        Map<String, List<GetBasePriceStrategyDetailOutput>> monthStrategyMap;
        List<GetBasePriceStrategyDetailOutput> monthStrategyList = null;
        List<BigDecimal> electricityCostList;
        List<BigDecimal> powerGenerationList;
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            //查最新间开历史
            if (this.isOptimize(context, detail.getId(), flag, startDate)) {
                continue;
            }
            //制度结束时间
            institutionEndDate = detail.getInstitutionEndDate();
            optimizeDeadline = DateUtil.offsetDay(startDate, offset);
            if (optimizeDeadline.compareTo(institutionEndDate) > 0) {
                optimizeDeadline = DateUtil.endOfDay(institutionEndDate);
            }
            //按是否需要优化分组
            if (!wellheadMap.containsKey(detail.getId())) {
                continue;
            }
            collect = wellheadMap.get(detail.getId()).stream()
                    .sorted(Comparator.comparing(SpaceInstitutionWellheadView::getStartSeq))
                    .collect(
                            Collectors.partitioningBy(w ->
                                    /*
                                    待优化井口:
                                        小间开,间抽井
                                    */
                                    CharSequenceUtil.equals(BusinessConstant.INTERVAL_PUMPING_WELL, w.getRunTypeKey()) &&
                                            CharSequenceUtil.equals("0", w.getIntervalTypeKey())
                            )
                    );
            //取待优化井口并遍历
            wellheadList = collect.get(Boolean.TRUE);
            //保存不需要优化的井口
            unOptimizeWellhead = collect.get(Boolean.FALSE);
            //电站ID列表
            plantIds = plantIdsMap.get(detail.getLineId());
            if (CollUtil.isEmpty(plantIds) || (CollUtil.isEmpty(wellheadList) && CollUtil.isEmpty(unOptimizeWellhead))) {
                //没有电站不优化
                continue;
            }
            //创建记录
            periodId = this.createOptimizePeriod(periodDTOList, detail.getId(), detail.getLineId(),
                    BaseUtils.getExecutionCycleForCalibration(startDate, DateUtil.offsetDay(optimizeDeadline, -1)), optimizeDeadline);
            //按天优化
            for (int d = 0; d < offset; d++) {
                begin = DateUtil.offsetDay(startDate, d);
                //取市电峰谷
                monthBaseOne = String.valueOf(begin.monthBaseOne());
                if (strategyMap.containsKey(monthBaseOne)) {
                    monthStrategyMap = strategyMap.get(monthBaseOne);
                    if (monthStrategyMap.containsKey(lineMap.get(detail.getLineId()))) {
                        monthStrategyList = monthStrategyMap.get(lineMap.get(detail.getLineId()));
                    }
                }
                if (CollUtil.isEmpty(monthStrategyList)) {
                    continue;
                }
                optimizeDate = DateUtil.offsetDay(startDate, d + 1);
                recordTime = offset > 1 ? begin : optimizeDate;
                //查预测光伏出力
                powerList = this.getAveragePower(context, DynamicQueryPlantPredictedPowerInput.builder()
                        .plantIds(plantIds)
                        .startTime(begin.toString(BusinessConstant.DATE_FORMAT_DAY))
                        .endTime(optimizeDate.toString(BusinessConstant.DATE_FORMAT_DAY))
                        .dateType(dateType)
                        .build());
                powerGenerationList = this.powerGenerationList(rangeToList, powerList);
                this.setUnOptimizeWellheadConfig(durationDTOList, unOptimizeDurationList, wellheadDTOList,
                        unOptimizeWellhead, durationMap.get(detail.getId()), periodId, recordTime);
                Integer tundraStrategy = detail.getTundraStrategy();
                //防冻堵条件:不存在常开井(不需要优化的井)&&需要优化的井口不能为空&&开启防冻堵
                if (CollUtil.isEmpty(unOptimizeWellhead) && CollUtil.isNotEmpty(wellheadList) && ObjectUtil.isNotNull(tundraStrategy) && tundraStrategy.equals(0)) {
                    int removeIndex = 0;
                    BigDecimal serviceRating = wellheadList.get(0).getServiceRating();
                    for (int i = 1; i < wellheadList.size(); i++) {
                        //找运行功率最小的井口(功率都一样取启动顺序最晚的)
                        if (serviceRating.compareTo(wellheadList.get(i).getServiceRating()) >= 0) {
                            SpaceInstitutionWellheadView wellhead = wellheadList.get(i);
                            String wellheadId = wellhead.getWellheadId();
                            String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber(), wellhead.getStartSeq(), recordTime, BusinessConstant.ZERO);
                            //保存间开原始记录
                            for (SpaceInstitutionDurationEnt durationEnt : durationMap.get(detail.getId()).get(wellheadId)) {
                                this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, recordTime);
                            }
                            //设置间开优化24小时全开
                            this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                    BusinessConstant.START_OF_DAY_TIME,
                                    BusinessConstant.END_OF_DAY_TIME,
                                    BusinessConstant.ZERO, recordTime
                            );
                            removeIndex = i;
                        }
                    }
                    //排除防冻井
                    wellheadList.remove(removeIndex);
                }
                SpaceInstitutionWellheadView wellhead;
                startInterval = detail.getStartInterval();
                if (CollUtil.isNotEmpty(wellheadList)) {
                    //计算权重
                    electricityCostList = this.electricityCostList(rangeToList, monthStrategyList);
                    Map<String, Object> objectObjectMap = this.setParams(powerGenerationList, electricityCostList, detail.getOptimizeType(), wellheadList);
                    String post = HttpUtil.post("http://127.0.0.1:10060/api/gaponoff/plan?secretKey=fZ1kC1aV8bO7rH3bL8oE5zI0mY3vW1sG", objectObjectMap);
                    List<OptimizeWellResult> resultList;
                    if (JSONUtil.isTypeJSON(post)) {
                        BaseResult result = JSONUtil.toBean(post, BaseResult.class);
                        //成功
                        if (result.getCode().equals(1)) {
                            String jsonStr = JSONUtil.toJsonStr(result.getData());
                            //判断是数组还是对象
                            if (JSONUtil.isTypeJSONArray(jsonStr)) {
                                resultList = JSONUtil.toList(jsonStr, OptimizeWellResult.class);
                            } else {
                                resultList = new ArrayList<>(2);
                                resultList.add(JSONUtil.toBean(jsonStr, OptimizeWellResult.class, true));
                            }
                            if (CollUtil.isEmpty(resultList)) {
                                continue;
                            }
                            Map<String, OptimizeWellResult> resultMap = resultList.stream()
                                    .collect(Collectors.toMap(OptimizeWellResult::getId, Function.identity()));
                            //遍历
                            for (int i = 0; i < wellheadList.size(); i++) {
                                wellhead = wellheadList.get(i);
                                String wellheadId = wellhead.getWellheadId();
                                //取优化结果
                                if (resultMap.containsKey(wellheadId)) {
                                    OptimizeWellResult optimizeWellResult = resultMap.get(wellheadId);
                                    List<OptimizeWellResult.OnOffDTO> onOffList = optimizeWellResult.getOnOff();
                                    if (CollUtil.isEmpty(onOffList)) {
                                        continue;
                                    }
                                    String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber(), wellhead.getStartSeq(), recordTime, BusinessConstant.ONE);
                                    //保存间开原始记录
                                    if (!durationMap.containsKey(detail.getId()) || !durationMap.get(detail.getId()).containsKey(wellheadId)) {
                                        continue;
                                    }
                                    for (SpaceInstitutionDurationEnt durationEnt : durationMap.get(detail.getId()).get(wellheadId)) {
                                        this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, recordTime);
                                    }
                                    onOffList = onOffList.stream()
                                            .filter(x -> CharSequenceUtil.equals(x.getStat(), BusinessConstant.OPEN_FLAG))
                                            .collect(Collectors.toList());
                                    List<OptimizeWellResult.OnOffDTO> tempOnOffList = new ArrayList<>(onOffList.size());
                                    tempOnOffList.add(onOffList.get(0));
                                    //优化处理结果
                                    for (int i1 = 1; i1 < onOffList.size(); i1++) {
                                        OptimizeWellResult.OnOffDTO nowOne = onOffList.get(i1);
                                        OptimizeWellResult.OnOffDTO lastOne = tempOnOffList.get(tempOnOffList.size() - 1);
                                        if (lastOne.getEndTime().equals(nowOne.getStartTime())) {
                                            lastOne.setEndTime(nowOne.getEndTime());
                                        } else {
                                            tempOnOffList.add(nowOne);
                                        }
                                    }
                                    //遍历创建开井时间段
                                    for (OptimizeWellResult.OnOffDTO onOff : tempOnOffList) {
                                        //只处理开井时段
                                        if (CharSequenceUtil.equals(BusinessConstant.OPEN_FLAG, onOff.getStat())) {
                                            DateTime startOffset = DateUtil.offsetMinute(BusinessConstant.DATE_FLAG_BEGIN, onOff.getStartTime());
                                            DateTime endOffset = DateUtil.offsetMinute(BusinessConstant.DATE_FLAG_BEGIN, onOff.getEndTime());
                                            if (startOffset.compareTo(BusinessConstant.DATE_FLAG) < 0 && endOffset.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                                                this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                                        startOffset.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                                                        BusinessConstant.ONE, recordTime
                                                );
                                                this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                                        BusinessConstant.START_OF_DAY_TIME, endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                                        BusinessConstant.ONE, recordTime
                                                );
                                            } else if (endOffset.compareTo(BusinessConstant.DATE_FLAG) == 0) {
                                                this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                                        startOffset.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                                                        BusinessConstant.ONE, recordTime
                                                );
                                            } else {
                                                this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                                        startOffset.toString(BusinessConstant.MINUTES_FORMAT), endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                                        BusinessConstant.ONE, recordTime
                                                );
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 动态优化
     *
     * @param context                上下文
     * @param periodDTOList          时期数据学家
     * @param wellheadDTOList        井口dtolist
     * @param durationDTOList        持续时间dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param dateType               日期类型
     * @param flag                   旗帜
     * @param offset                 抵消
     */
    public void dynamicOptimize(XContext context, List<SpaceOptimizePeriodDTO> periodDTOList, List<SpaceOptimizeWellheadDTO> wellheadDTOList,
                                List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                Integer dateType, Integer flag, Integer offset) {
        //取当前时间
        DateTime startDate = DateUtil.beginOfDay(DateTime.now());
        //取生效中的基础间开
        List<SpaceInstitutionDetailEnt> detailEntList = this.getEffectiveSpaceInstitution(context, startDate);
        //取线路ID和制度ID
        List<String> lineIds = new ArrayList<>(32);
        List<String> institutionIds = new ArrayList<>(32);
        detailEntList.forEach(detail -> {
            lineIds.add(detail.getLineId());
            institutionIds.add(detail.getId());
        });
        //取间开的所有井口
        List<SpaceInstitutionWellheadView> spaceWellheadList = this.getSpaceWellheadList(context, institutionIds);
        if (CollUtil.isEmpty(spaceWellheadList)) {
            throw new XServiceException(BusinessError.DidNotFindWellhead);
        }
        //取间开对应的线路
        List<DynamicQueryBasePowerLineOutput> powerLineList = ServiceUtil.getPowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .lineIds(lineIds)
                .build());
        //查线路下所有电站
        List<DynamicQueryBasePowerLinePlantViewOutput> powerLinePlantList = ServiceUtil.getPowerLinePlantList(context, DynamicQueryBasePowerLinePlantInput.builder()
                .lineIds(lineIds)
                .build());
        //取当前月
        int month = startDate.monthBaseOne();
        //查井口配置时间段
        SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
        List<SpaceInstitutionDurationEnt> durationList = durationMapper.selectList(new QueryWrapper<SpaceInstitutionDurationEnt>()
                .select("*",
                        "STR_TO_DATE( CONCAT( open_well_time, ':00' ), '%H:%i:%s' ) AS openTime",
                        "STR_TO_DATE( CONCAT( close_well_time, ':00' ), '%H:%i:%s' ) AS closeTime"
                )
                .lambda()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .in(SpaceInstitutionDurationEnt::getWellheadId, spaceWellheadList.stream().map(SpaceInstitutionWellheadView::getWellheadId).collect(Collectors.toList()))
                .orderByAsc(SpaceInstitutionDurationEnt::getOpenWellTime)
        );
        if (CollUtil.isEmpty(durationList)) {
            throw new XServiceException(BusinessError.DidNotFindCalibration);
        }
        //间开制度ID/井口LIST Map
        Map<String, List<SpaceInstitutionWellheadView>> wellheadMap = spaceWellheadList.stream()
                .collect(Collectors.groupingBy(SpaceInstitutionWellheadView::getInstitutionId));
        //线路ID/峰谷策略ID Map
        Map<String, String> lineMap = powerLineList.stream()
                .collect(Collectors.toMap(DynamicQueryBasePowerLineOutput::getId, DynamicQueryBasePowerLineOutput::getStrategyId));
        //线路ID/电站ID LIST Map
        Map<String, List<String>> plantIdsMap = powerLinePlantList.stream()
                .collect(Collectors.groupingBy(DynamicQueryBasePowerLinePlantViewOutput::getLineId,
                        Collectors.mapping(DynamicQueryBasePowerLinePlantViewOutput::getPlantId, Collectors.toList())));
        //峰谷策略ID/峰谷策略LIST Map
        Map<String, List<GetBasePriceStrategyDetailOutput>> strategyMap = ServiceUtil.getStrategyDetailList(context, GetBasePriceStrategyDetailInput.builder()
                        .strategyMonth(String.valueOf(month))
                        .build()).stream()
                .collect(Collectors.groupingBy(GetBasePriceStrategyDetailOutput::getStrategyId));
        //间开制度ID/区间配置Map Map
        Map<String, Map<String, List<SpaceInstitutionDurationEnt>>> durationMap = durationList.stream()
                .collect(Collectors.groupingBy(SpaceInstitutionDurationEnt::getInstitutionId,
                        Collectors.groupingBy(SpaceInstitutionDurationEnt::getWellheadId)));
        //初始化时间轴
        List<DateTime> rangeToList = DateUtil.rangeToList(DateUtil.offsetDay(BusinessConstant.DATE_FLAG, -1), BusinessConstant.DATE_FLAG, DateField.MINUTE, 30);
        //初始化时间轴(5分钟间隔)
        List<DateTime> timerShaft_5 = DateUtil.rangeToList(DateUtil.offsetDay(BusinessConstant.DATE_FLAG, -1), BusinessConstant.DATE_FLAG, DateField.MINUTE, 5);
        //遍历
        List<DynamicQueryPlantPredictedPowerOutput> powerList;
        List<GetBasePriceStrategyDetailOutput> strategyList;
        List<SpaceInstitutionWellheadView> wellheadList;
        List<SpaceInstitutionWellheadView> unOptimizeWellhead;
        List<String> plantIds;
        String periodId;
        Map<Boolean, List<SpaceInstitutionWellheadView>> collect;
        //时间间隔
        Integer startInterval;
        Date institutionEndDate;
        DateTime optimizeDeadline;
        //取优化截至日期
        DateTime optimizeDate;
        DateTime begin;
        DateTime recordTime;
        boolean isOffGrid;
        List<SpaceOptimizeDurationDTO> offGridDTOList = null;
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            //是否离网型线路
            isOffGrid = CharSequenceUtil.equals(detail.getGridTypeKey(), "0");
            //查最新间开历史
            if (this.isOptimize(context, detail.getId(), flag, startDate)) {
                continue;
            }
            //制度结束时间
            institutionEndDate = detail.getInstitutionEndDate();
            optimizeDeadline = DateUtil.offsetDay(startDate, offset);
            if (optimizeDeadline.compareTo(institutionEndDate) > 0) {
                optimizeDeadline = DateUtil.endOfDay(institutionEndDate);
            }
            //取峰谷策略
            strategyList = strategyMap.get(lineMap.get(detail.getLineId()));
            //按是否需要优化分组
            if (!wellheadMap.containsKey(detail.getId())) {
                continue;
            }
            collect = wellheadMap.get(detail.getId()).stream()
                    .sorted(Comparator.comparing(SpaceInstitutionWellheadView::getStartSeq))
                    .collect(
                            Collectors.partitioningBy(w ->
                                    /*
                                    待优化井口:
                                        小间开,间抽井,
                                        最小开井时间<=每日运行时间,
                                        运行时长>=单次最高开井时长>=单次最低开井时长,
                                        单次最低开井时长+单次最低关井时长<=24
                                    */
                                    CharSequenceUtil.equals(BusinessConstant.INTERVAL_PUMPING_WELL, w.getRunTypeKey()) &&
                                            CharSequenceUtil.equals("0", w.getIntervalTypeKey()) &&
                                            w.getRunDuration().compareTo(w.getMinOpen()) >= 0 &&
                                            w.getRunDuration().compareTo(w.getMaxOpen()) >= 0 &&
                                            w.getMaxOpen().compareTo(w.getMinOpen()) >= 0 &&
                                            BusinessConstant.BIG_DECIMAL_24.compareTo(w.getMinOpen().add(w.getMinClose())) >= 0
                            )
                    );
            //取待优化井口并遍历
            wellheadList = collect.get(Boolean.TRUE);
            //保存不需要优化的井口
            unOptimizeWellhead = collect.get(Boolean.FALSE);
            //电站ID列表
            plantIds = plantIdsMap.get(detail.getLineId());
            if (CollUtil.isEmpty(plantIds) || (CollUtil.isEmpty(wellheadList) && CollUtil.isEmpty(unOptimizeWellhead))) {
                //没有电站不优化
                continue;
            }
            //创建记录
            periodId = this.createOptimizePeriod(periodDTOList, detail.getId(), detail.getLineId(),
                    BaseUtils.getExecutionCycleForCalibration(startDate, DateUtil.offsetDay(optimizeDeadline, -1)), optimizeDeadline);
            //按天优化
            for (int d = 0; d < offset; d++) {
                begin = DateUtil.offsetDay(startDate, d);
                optimizeDate = DateUtil.offsetDay(startDate, d + 1);
                recordTime = offset > 1 ? begin : optimizeDate;
                //查预测光伏出力
                powerList = this.getAveragePower(context, DynamicQueryPlantPredictedPowerInput.builder()
                        .plantIds(plantIds)
                        .startTime(begin.toString(BusinessConstant.DATE_FORMAT_DAY))
                        .endTime(optimizeDate.toString(BusinessConstant.DATE_FORMAT_DAY))
                        .dateType(dateType)
                        .build());
                this.setUnOptimizeWellheadConfig(durationDTOList, unOptimizeDurationList, wellheadDTOList,
                        unOptimizeWellhead, durationMap.get(detail.getId()), periodId, recordTime);
                //防冻堵策略(一口井就全开不走优化)
                SpaceInstitutionWellheadView tundraStrategyWellhead = null;
                List<SpaceOptimizeDurationDTO> tundraStrategyList = new ArrayList<>(32);
                Integer tundraStrategy = detail.getTundraStrategy();
                //防冻堵条件:不存在常开井(不需要优化的井)&&需要优化的井口不能为空&&开启防冻堵
                if (CollUtil.isEmpty(unOptimizeWellhead) && CollUtil.isNotEmpty(wellheadList) && ObjectUtil.isNotNull(tundraStrategy) && tundraStrategy.equals(0)) {
                    int removeIndex = 0;
                    BigDecimal serviceRating = wellheadList.get(0).getServiceRating();
                    if (wellheadList.size() == 1) {
                        SpaceInstitutionWellheadView wellhead = wellheadList.get(0);
                        String wellheadId = wellhead.getWellheadId();
                        String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber(), wellhead.getStartSeq(), recordTime, BusinessConstant.ZERO);
                        //保存间开原始记录
                        for (SpaceInstitutionDurationEnt durationEnt : durationMap.get(detail.getId()).get(wellheadId)) {
                            this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, recordTime);
                        }
                        //设置间开优化24小时全开
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                BusinessConstant.START_OF_DAY_TIME,
                                BusinessConstant.END_OF_DAY_TIME,
                                BusinessConstant.ZERO, recordTime
                        );
                    } else {
                        for (int i = 1; i < wellheadList.size(); i++) {
                            //找运行功率最小的井口(功率都一样取启动顺序最晚的)
                            if (serviceRating.compareTo(wellheadList.get(i).getServiceRating()) >= 0) {
                                removeIndex = i;
                            }
                        }
                        //取出防冻井
                        tundraStrategyWellhead = wellheadList.get(removeIndex);
                    }
                    //排除防冻井
                    wellheadList.remove(removeIndex);
                }
                BigDecimal serviceRating = BigDecimal.ZERO;
                SpaceInstitutionWellheadView wellhead;
                //记录第一次开井时间
                DateTime firstOpenWellTime = null;
                startInterval = detail.getStartInterval();
                if (CollUtil.isNotEmpty(wellheadList)) {
                    for (int i = 0; i < wellheadList.size(); i++) {
                        if (isOffGrid) {
                            offGridDTOList = new ArrayList<>(32);
                        }
                        wellhead = wellheadList.get(i);
                        String wellheadId = wellhead.getWellheadId();
                        String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber(), wellhead.getStartSeq(), recordTime, BusinessConstant.ONE);
                        //保存间开原始记录
                        if (!durationMap.containsKey(detail.getId()) || !durationMap.get(detail.getId()).containsKey(wellheadId)) {
                            continue;
                        }
                        for (SpaceInstitutionDurationEnt durationEnt : durationMap.get(detail.getId()).get(wellheadId)) {
                            this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, recordTime);
                        }
                        //累加运行功率
                        serviceRating = serviceRating.add(wellhead.getServiceRating());
                        //计算权重
                        List<SpaceOptimizeWeight> weightList = this.calculateWeightList(serviceRating, rangeToList, powerList, strategyList, isOffGrid);
                        //每日最小开井时长(分钟)
                        int minOpenMinute = wellhead.getMinOpen()
                                .multiply(BusinessConstant.BIG_DECIMAL_60)
                                .intValue();
                        //分级取时间段
                        List<SpaceOptimizeWeightDuration> weightDurationList = this.weightListProcessing(weightList, minOpenMinute);
                        //时间段优化
                        List<SpaceOptimizeDurationDTO> optimizeDurationDTOList = new ArrayList<>(12);
                        firstOpenWellTime = this.timePeriodOptimization(optimizeDurationDTOList, weightDurationList, firstOpenWellTime, i,
                                startInterval, minOpenMinute, wellhead, weightList);
                        //创建优化后的间开区间
                        if (isOffGrid) {
                            this.addOptimizeDuration(optimizeDurationDTOList, offGridDTOList, tundraStrategyList, periodId, recordId, wellheadId, recordTime);
                            //标注发电类型
                            this.addOffGridOptimizeDuration(durationDTOList, offGridDTOList, weightList);
                        } else {
                            this.addOptimizeDuration(optimizeDurationDTOList, durationDTOList, tundraStrategyList, periodId, recordId, wellheadId, recordTime);
                        }
                    }
                    //防冻井
                    if (ObjectUtil.isNotNull(tundraStrategyWellhead) && CollUtil.isNotEmpty(tundraStrategyList)) {
                        List<SpaceOptimizeDurationDTO> closeList = new ArrayList<>(32);
                        long totalOpenTime = 0;
                        if (tundraStrategyList.size() == 1) {
                            //记录第一段开始时间之前的关井时间
                            if (BusinessConstant.DATE_FLAG_BEGIN.compareTo(tundraStrategyList.get(0).getOpenTime()) != 0) {
                                long between = DateUtil.between(BusinessConstant.DATE_FLAG_BEGIN, tundraStrategyList.get(0).getOpenTime(), DateUnit.MINUTE);
                                totalOpenTime += between;
                                closeList.add(
                                        SpaceOptimizeDurationDTO.builder()
                                                .openTime(BusinessConstant.DATE_FLAG_BEGIN)
                                                .closeTime(tundraStrategyList.get(0).getOpenTime())
                                                .duration(between)
                                                .build()
                                );
                            }
                            //记录最后一段停井时间之后的关井时间
                            if (BusinessConstant.DATE_FLAG.compareTo(tundraStrategyList.get(0).getCloseTime()) != 0) {
                                long between = DateUtil.between(tundraStrategyList.get(0).getCloseTime(), BusinessConstant.DATE_FLAG, DateUnit.MINUTE);
                                totalOpenTime += between;
                                closeList.add(
                                        SpaceOptimizeDurationDTO.builder()
                                                .openTime(tundraStrategyList.get(0).getCloseTime())
                                                .closeTime(BusinessConstant.DATE_FLAG)
                                                .duration(between)
                                                .build()
                                );
                            }
                        } else {
                            //初始化时间轴
                            List<SpaceOptimizeTundraStrategy> axis = new ArrayList<>(timerShaft_5.size());
                            for (DateTime dateTime : timerShaft_5) {
                                axis.add(
                                        SpaceOptimizeTundraStrategy.builder()
                                                .timestamp(dateTime)
                                                .beUsing(Boolean.FALSE)
                                                .build()
                                );
                            }
                            //先按开井时间再按关井时间排序
                            tundraStrategyList.sort(Comparator.comparing(SpaceOptimizeDurationDTO::getOpenTime).thenComparing(SpaceOptimizeDurationDTO::getCloseTime));
                            //渲染时间轴
                            int nextBegin = 0;
                            for (SpaceOptimizeDurationDTO durationDTO : tundraStrategyList) {
                                Date openTime = durationDTO.getOpenTime();
                                Date closeTime = durationDTO.getCloseTime();
                                for (int i = nextBegin; i < axis.size(); i++) {
                                    SpaceOptimizeTundraStrategy z = axis.get(i);
                                    if (DateUtil.isIn(z.getTimestamp(), openTime, closeTime)) {
                                        z.setBeUsing(Boolean.TRUE);
                                    }
                                    //记录下一次开始位置
                                    if (z.getTimestamp().compareTo(closeTime) > 0) {
                                        nextBegin = i;
                                        break;
                                    }
                                }
                            }
                            //取关井时间段
                            int beginIndex = 0;
                            boolean isClose = Boolean.FALSE;
                            for (int i = 0; i < axis.size(); i++) {
                                SpaceOptimizeTundraStrategy z = axis.get(i);
                                Boolean beUsing = z.getBeUsing();
                                if (isClose && beUsing) {
                                    long between = DateUtil.between(axis.get(beginIndex).getTimestamp(), axis.get(i).getTimestamp(), DateUnit.MINUTE);
                                    totalOpenTime += between;
                                    //存时间段,重置状态
                                    closeList.add(
                                            SpaceOptimizeDurationDTO.builder()
                                                    .openTime(axis.get(beginIndex).getTimestamp())
                                                    .closeTime(axis.get(i).getTimestamp())
                                                    .duration(between)
                                                    .build()
                                    );
                                    isClose = Boolean.FALSE;
                                } else if (!isClose && !beUsing) {
                                    //确定开始位置
                                    if (i > 0) {
                                        beginIndex = i - 1;
                                    }
                                    isClose = Boolean.TRUE;
                                }
                            }
                            //补充最后一段
                            if (isClose) {
                                long between = DateUtil.between(axis.get(beginIndex).getTimestamp(), axis.get(axis.size() - 1).getTimestamp(), DateUnit.MINUTE);
                                totalOpenTime += between;
                                closeList.add(
                                        SpaceOptimizeDurationDTO.builder()
                                                .openTime(axis.get(beginIndex).getTimestamp())
                                                .closeTime(axis.get(axis.size() - 1).getTimestamp())
                                                .duration(between)
                                                .build()
                                );
                            }
                        }
                        //计算权重时间轴
                        //每日最小开井时长(分钟)
                        int minOpenMinute = tundraStrategyWellhead.getMinOpen()
                                .multiply(BusinessConstant.BIG_DECIMAL_60)
                                .intValue();
                        serviceRating = serviceRating.add(tundraStrategyWellhead.getServiceRating());
                        List<SpaceOptimizeWeight> weightList = this.calculateWeightList(serviceRating, rangeToList, powerList, strategyList, isOffGrid);
                        List<SpaceOptimizeWeightDuration> weightDurationList = this.weightListProcessing(weightList, minOpenMinute);
                        //创建优化记录
                        String wellheadId = tundraStrategyWellhead.getWellheadId();
                        String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, tundraStrategyWellhead.getWellNumber(), tundraStrategyWellhead.getStartSeq(), recordTime, BusinessConstant.ZERO);
                        //保存间开原始记录
                        for (SpaceInstitutionDurationEnt durationEnt : durationMap.get(detail.getId()).get(wellheadId)) {
                            this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, recordTime);
                        }
                        //关井时间优化
                        if (CollUtil.isNotEmpty(closeList)) {
                            //每日开井时长
                            int dayOpenMinute = tundraStrategyWellhead.getRunDuration()
                                    .multiply(BusinessConstant.BIG_DECIMAL_60)
                                    .intValue();
                            //计算需要补的时间
                            long supplementaryTime = dayOpenMinute - totalOpenTime;
                            //补时间
                            for (SpaceOptimizeWeightDuration weightDuration : weightDurationList) {
                                if (supplementaryTime <= 0) {
                                    break;
                                }
                                DateTime openTimeWeight = weightDuration.getOpenTime();
                                DateTime closeTimeWeight = weightDuration.getCloseTime();
                                //计算排除次数
                                int exceptSize = 0;
                                for (int i = 0; i < closeList.size(); i++) {
                                    SpaceOptimizeDurationDTO close = closeList.get(i);
                                    Date openTime = close.getOpenTime();
                                    Date closeTime = close.getCloseTime();
                                    if (DateUtil.compare(openTime, openTimeWeight) >= 0 && DateUtil.compare(closeTimeWeight, closeTime) >= 0) {
                                        //可优化时间段在开井范围内
                                        if (i > 0) {
                                            //判断开井时间是否小于上一段关井时间
                                            if (DateUtil.compare(openTimeWeight, closeList.get(i - 1).getCloseTime()) < 0) {
                                                //如果小于上一段关井时间,修改当前开井时间
                                                openTimeWeight = DateUtil.date(closeList.get(i - 1).getCloseTime());
                                            }
                                        }
                                        //取后一位
                                        if (i + 1 < closeList.size()) {
                                            //判断关井时间是否大于下一段开井时间
                                            if (DateUtil.compare(closeTimeWeight, closeList.get(i + 1).getOpenTime()) > 0) {
                                                //如果大于下一段开井时间,修改当前关井时间
                                                closeTimeWeight = DateUtil.date(closeList.get(i + 1).getOpenTime());
                                            }
                                        }
                                        //这里需要把时长补回去再计算
                                        supplementaryTime += close.getDuration();
                                        //重新计算结束时间及总开井时长
                                        long between = DateUtil.between(openTimeWeight, closeTimeWeight, DateUnit.MINUTE);
                                        if (between >= supplementaryTime) {
                                            //从后往前计算开井时间
                                            close.setDuration(supplementaryTime);
                                            supplementaryTime = 0;
                                        } else {
                                            close.setDuration(between);
                                            supplementaryTime -= between;
                                        }
                                        close.setOpenTime(openTimeWeight);
                                        close.setCloseTime(closeTimeWeight);
                                        break;
                                    }
                                    if (DateUtil.compare(openTime, closeTimeWeight) >= 0 || DateUtil.compare(closeTime, openTimeWeight) <= 0) {
                                        //判断时间段是否在开井范围之外
                                        exceptSize++;
                                        continue;
                                    }
                                    //从后往前算
                                    if (DateUtil.isIn(openTime, openTimeWeight, closeTimeWeight)) {
                                        //取前一位
                                        if (i > 0) {
                                            //判断开井时间是否小于上一段关井时间
                                            if (DateUtil.compare(openTimeWeight, closeList.get(i - 1).getCloseTime()) < 0) {
                                                //如果小于上一段关井时间,修改当前开井时间
                                                openTimeWeight = DateUtil.date(closeList.get(i - 1).getCloseTime());
                                            }
                                        }
                                        //这里需要把时长补回去再计算
                                        supplementaryTime += close.getDuration();
                                        //重新计算结束时间及总开井时长
                                        long between = DateUtil.between(openTimeWeight, closeTime, DateUnit.MINUTE);
                                        if (between >= supplementaryTime) {
                                            //从后往前计算开井时间
                                            close.setOpenTime(DateUtil.offsetMinute(closeTime, -(int) supplementaryTime));
                                            close.setDuration(supplementaryTime);
                                            supplementaryTime = 0;
                                            break;
                                        } else {
                                            close.setOpenTime(openTimeWeight);
                                            close.setDuration(between);
                                            supplementaryTime -= between;
                                        }
                                    }
                                    //从前往后算
                                    if (DateUtil.isIn(closeTime, openTimeWeight, closeTimeWeight)) {
                                        //取后一位
                                        if (i + 1 < closeList.size()) {
                                            //判断关井时间是否大于下一段开井时间
                                            if (DateUtil.compare(closeTimeWeight, closeList.get(i + 1).getOpenTime()) > 0) {
                                                //如果大于下一段开井时间,修改当前关井时间
                                                closeTimeWeight = DateUtil.date(closeList.get(i + 1).getOpenTime());
                                            }
                                        }
                                        //这里需要把时长补回去再计算
                                        supplementaryTime += close.getDuration();
                                        //重新计算开始时间及总开井时长
                                        long between = DateUtil.between(openTime, closeTimeWeight, DateUnit.MINUTE);
                                        if (between >= supplementaryTime) {
                                            //从后往前计算开井时间
                                            close.setCloseTime(DateUtil.offsetMinute(openTime, (int) supplementaryTime));
                                            close.setDuration(supplementaryTime);
                                            supplementaryTime = 0;
                                            break;
                                        } else {
                                            close.setCloseTime(closeTimeWeight);
                                            close.setDuration(between);
                                            supplementaryTime -= between;
                                        }
                                    }
                                }
                                //添加时段,处理开井时长
                                if (exceptSize == closeList.size()) {
                                    long duration = weightDuration.getDuration();
                                    if (duration >= supplementaryTime) {
                                        //计算关井时间
                                        closeTimeWeight = DateUtil.offsetMinute(openTimeWeight, (int) supplementaryTime);
                                        duration = supplementaryTime;
                                        supplementaryTime = 0;
                                    } else {
                                        supplementaryTime -= duration;
                                    }
                                    closeList.add(
                                            SpaceOptimizeDurationDTO.builder()
                                                    .openTime(openTimeWeight)
                                                    .closeTime(closeTimeWeight)
                                                    .duration(duration)
                                                    .build()
                                    );
                                }
                            }
                            //时间段合并优化
                            closeList.sort(Comparator.comparing(SpaceOptimizeDurationDTO::getOpenTime));
                            List<SpaceOptimizeDurationDTO> optimizeDurationDTOList = new ArrayList<>(closeList.size());
                            optimizeDurationDTOList.add(closeList.get(0));
                            for (int i = 1; i < closeList.size(); i++) {
                                SpaceOptimizeDurationDTO nowOne = closeList.get(i);
                                SpaceOptimizeDurationDTO lastOne = optimizeDurationDTOList.get(optimizeDurationDTOList.size() - 1);
                                if (0 == DateUtil.compare(lastOne.getCloseTime(), nowOne.getOpenTime())) {
                                    lastOne.setCloseTime(nowOne.getCloseTime());
                                } else {
                                    optimizeDurationDTOList.add(nowOne);
                                }
                            }
                            closeList = optimizeDurationDTOList;
                        } else {
                            //不需要补时间,需要优化计算
                            this.timePeriodOptimization(closeList, weightDurationList, firstOpenWellTime, wellheadList.size(),
                                    startInterval, minOpenMinute, tundraStrategyWellhead, weightList);
                        }
                        //创建优化后的间开区间
                        this.addOptimizeDuration(closeList, durationDTOList, tundraStrategyList, periodId, recordId, wellheadId, recordTime);
                    }
                }
            }
        }
    }

    /**
     * 进行优化
     *
     * @param context                上下文
     * @param periodDTOList          时期数据学家
     * @param wellheadDTOList        井口dtolist
     * @param durationDTOList        持续时间dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param dateType               日期类型
     * @param flag                   旗帜
     * @param offset                 抵消
     */
    public void doOptimize(XContext context, List<SpaceOptimizePeriodDTO> periodDTOList, List<SpaceOptimizeWellheadDTO> wellheadDTOList,
                           List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                           Integer dateType, Integer flag, Integer offset) {
        //取当前时间
        DateTime date = DateUtil.beginOfDay(DateTime.now());
        //取生效中的基础间开
        List<SpaceInstitutionDetailEnt> detailEntList = this.getEffectiveSpaceInstitution(context, date);
        SpaceOptimizeLineRelation relation = this.getOptimizeLineRelation(context, detailEntList);
        //月份要加一
        int monthNum = date.month() + 1;
        //优化
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            //查最新间开历史
            if (this.isOptimize(context, detail.getId(), flag, date)) {
                continue;
            }
            Date institutionEndDate = detail.getInstitutionEndDate();
            DateTime optimizeDeadline = DateUtil.offsetDay(date, offset);
            if (optimizeDeadline.compareTo(institutionEndDate) > 0) {
                optimizeDeadline = DateUtil.endOfDay(institutionEndDate);
            }
            //创建记录
            String periodId = this.createOptimizePeriod(periodDTOList, detail.getId(), detail.getLineId(), BaseUtils.getExecutionCycleForCalibration(date, DateUtil.offsetDay(optimizeDeadline, -1)), optimizeDeadline);
            for (int i = 0; i < offset; i++) {
                DateTime startDate = DateUtil.offsetDay(date, i);
                DateTime optimizeDate = DateUtil.offsetDay(date, i + 1);
                DynamicQueryPlantPredictedPowerInput powerInput = DynamicQueryPlantPredictedPowerInput.builder()
                        .plantIds(relation.getPlantIdsByLineIdMap().get(detail.getLineId()))
                        .startTime(startDate.toString(BusinessConstant.DATE_FORMAT_DAY))
                        .endTime(optimizeDate.toString(BusinessConstant.DATE_FORMAT_DAY))
                        .dateType(dateType)
                        .formatType(0)
                        .build();
                switch (detail.getGridTypeKey()) {
                    //并网型优化
                    case "1":
                        this.gridConnectedOptimization(context, wellheadDTOList, durationDTOList, unOptimizeDurationList,
                                relation.getWellheadsByInstitutionIdIdMap().get(detail.getId()), detail, monthNum, periodId,
                                powerInput, offset > 1 ? startDate : optimizeDate
                        );
                        break;
                    //离网型优化
                    case "0":
                        this.offGridOptimization(context, wellheadDTOList, durationDTOList, unOptimizeDurationList,
                                relation.getWellheadsByInstitutionIdIdMap().get(detail.getId()), relation.getStorageAvgMap(), detail, periodId,
                                powerInput, offset > 1 ? startDate : optimizeDate
                        );
                        break;
                    default:
                        //电网类型不存在
                }
            }
        }
    }

    /**
     * 计算优化效果:
     * 1.离网只算绿电占比
     * 2.不需要优化的井,也需要计算
     *
     * @param context                上下文
     * @param periodDTOList          周期数据列表
     * @param wellheadDTOList        井口数据表
     * @param durationDTOList        优化结果
     * @param unOptimizeDurationList 未优化持续时间列表
     * @param offset                 偏移
     * @param dateType               数据范围
     */
    public void calculateOptimizeEffect(XContext context, List<SpaceOptimizePeriodDTO> periodDTOList, List<SpaceOptimizeWellheadDTO> wellheadDTOList,
                                        List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                        Integer offset, Integer dateType) {
        if (CollUtil.isNotEmpty(periodDTOList)) {
            //按周期ID分组
            Map<String, Map<Date, List<SpaceOptimizeWellheadDTO>>> wellMap;
            if (CollUtil.isNotEmpty(wellheadDTOList)) {
                wellMap = wellheadDTOList.stream()
                        .collect(Collectors.groupingBy(SpaceOptimizeWellheadDTO::getPeriodId,
                                Collectors.groupingBy(SpaceOptimizeWellheadDTO::getOptimizeDate)));
            } else {
                wellMap = Collections.emptyMap();
            }
            Map<String, Map<String, Map<Date, List<SpaceOptimizeDurationDTO>>>> optimizeMap;
            if (CollUtil.isNotEmpty(durationDTOList)) {
                optimizeMap = durationDTOList.stream()
                        .collect(Collectors.groupingBy(SpaceOptimizeDurationDTO::getPeriodId,
                                Collectors.groupingBy(SpaceOptimizeDurationDTO::getWellheadId,
                                        Collectors.groupingBy(SpaceOptimizeDurationDTO::getOptimizeDate))));
            } else {
                optimizeMap = Collections.emptyMap();
            }
            Map<String, Map<String, Map<Date, List<SpaceOptimizeDurationDTO>>>> unOptimizeMap;
            if (CollUtil.isNotEmpty(unOptimizeDurationList)) {
                unOptimizeMap = unOptimizeDurationList.stream()
                        .collect(Collectors.groupingBy(SpaceOptimizeDurationDTO::getPeriodId,
                                Collectors.groupingBy(SpaceOptimizeDurationDTO::getWellheadId,
                                        Collectors.groupingBy(SpaceOptimizeDurationDTO::getOptimizeDate))));
            } else {
                unOptimizeMap = Collections.emptyMap();
            }
            DateTime now = DateUtil.beginOfDay(DateUtil.date());
            int month = now.monthBaseOne();
            //市电峰谷
            Map<String, Map<String, List<GetBasePriceStrategyDetailOutput>>> strategyMap;
            List<GetBasePriceStrategyDetailOutput> strategyList = ServiceUtil.getStrategyDetailList(context, GetBasePriceStrategyDetailInput.builder()
                    .monthList(CollUtil.list(false, String.valueOf(month), String.valueOf(month + 1)))
                    .build());
            if (CollUtil.isNotEmpty(strategyList)) {
                strategyMap = strategyList.stream()
                        .collect(Collectors.groupingBy(GetBasePriceStrategyDetailOutput::getStrategyMonth,
                                Collectors.groupingBy(GetBasePriceStrategyDetailOutput::getStrategyId)));
            } else {
                strategyMap = Collections.emptyMap();
            }
            //线路 Map
            Map<String, DynamicQueryBasePowerLineOutput> lineMap;
            //取线路列表
            List<DynamicQueryBasePowerLineOutput> powerLineList = ServiceUtil.getPowerLineList(context, new DynamicQueryBasePowerLineInput());
            if (CollUtil.isNotEmpty(powerLineList)) {
                lineMap = powerLineList.stream()
                        .collect(Collectors.toMap(DynamicQueryBasePowerLineOutput::getId, Function.identity()));
            } else {
                lineMap = Collections.emptyMap();
            }
            //线路ID/电站ID LIST Map
            Map<String, List<String>> plantIdsMap;
            //查关联线路的所有电站
            List<DynamicQueryBasePowerLinePlantViewOutput> powerLinePlantList = ServiceUtil.getPowerLinePlantList(context, DynamicQueryBasePowerLinePlantInput.builder()
                    .build());
            if (CollUtil.isNotEmpty(powerLinePlantList)) {
                plantIdsMap = powerLinePlantList.stream()
                        .collect(Collectors.groupingBy(DynamicQueryBasePowerLinePlantViewOutput::getLineId,
                                Collectors.mapping(DynamicQueryBasePowerLinePlantViewOutput::getPlantId, Collectors.toList())));
            } else {
                plantIdsMap = Collections.emptyMap();
            }
            //取所有间开关联井口
            Map<String, SpaceInstitutionWellheadView> baseWellMap;
            List<SpaceInstitutionWellheadView> wellList = this.getSpaceWellheadList(context, null);
            if (CollUtil.isNotEmpty(wellList)) {
                baseWellMap = wellList.stream()
                        .collect(Collectors.toMap(SpaceInstitutionWellheadView::getWellheadId, Function.identity()));
            } else {
                baseWellMap = Collections.emptyMap();
            }
            //初始化权重时间轴(15分钟间隔)
            List<DateTime> rangeToList = DateUtil.rangeToList(DateUtil.offsetDay(BusinessConstant.DATE_FLAG, -1), BusinessConstant.DATE_FLAG, DateField.MINUTE, 15);
            //遍历
            DateTime begin;
            Map<Date, List<SpaceOptimizeWellheadDTO>> wellDateMap;
            DynamicQueryBasePowerLineOutput line;
            List<String> plantIds;
            List<DynamicQueryPlantPredictedPowerOutput> powerList;
            String monthBaseOne;
            Map<String, List<GetBasePriceStrategyDetailOutput>> monthStrategyMap;
            List<GetBasePriceStrategyDetailOutput> monthStrategyList = null;
            Map<String, BigDecimal> electrovalenceMap;
            SpaceInstitutionWellheadView baseWell;
            BigDecimal serviceRating;
            List<SpaceOptimizeWeight> weightList;
            Map<String, Map<Date, List<SpaceOptimizeDurationDTO>>> periodDurationMap;
            Map<Date, List<SpaceOptimizeDurationDTO>> durationMap;
            List<SpaceOptimizeDurationDTO> durationList;
            DateTime timestamp;
            DateTime openTime;
            DateTime closeTime;
            StringBuilder optimalResult = new StringBuilder();
            int weightIndex;
            SpaceOptimizeWeight weight;
            SpaceOptimizeEffectView effectView;
            List<SpaceOptimizeEffectView> effectList = new ArrayList<>(baseWellMap.size() * offset);
            for (SpaceOptimizePeriodDTO period : periodDTOList) {
                //取线路
                if (!lineMap.containsKey(period.getLineId())) {
                    continue;
                }
                line = lineMap.get(period.getLineId());
                //取电站ID列表
                plantIds = plantIdsMap.get(period.getLineId());
                for (int d = 0; d <= offset; d++) {
                    begin = DateUtil.offsetDay(now, d);
                    //取市电峰谷
                    monthBaseOne = String.valueOf(begin.monthBaseOne());
                    if (strategyMap.containsKey(monthBaseOne)) {
                        monthStrategyMap = strategyMap.get(monthBaseOne);
                        if (monthStrategyMap.containsKey(line.getStrategyId())) {
                            monthStrategyList = monthStrategyMap.get(line.getStrategyId());
                        }
                    }
                    if (CollUtil.isEmpty(monthStrategyList)) {
                        continue;
                    }
                    //取电价
                    electrovalenceMap = new HashMap<>(4);
                    for (GetBasePriceStrategyDetailOutput strategy : monthStrategyList) {
                        electrovalenceMap.put(strategy.getPeriodTypeKey(), strategy.getElectrovalence());
                    }
                    //取优化井
                    if (!wellMap.containsKey(period.getId())) {
                        continue;
                    }
                    wellDateMap = wellMap.get(period.getId());
                    if (!wellDateMap.containsKey(begin)) {
                        continue;
                    }
                    //查光伏预测
                    powerList = this.getAveragePower(context, DynamicQueryPlantPredictedPowerInput.builder()
                            .plantIds(plantIds)
                            .startTime(begin.toString(BusinessConstant.DATE_FORMAT_DAY))
                            .endTime(DateUtil.offsetDay(begin, 1).toString(BusinessConstant.DATE_FORMAT_DAY))
                            .dateType(dateType)
                            .formatType(0)
                            .build());
                    //遍历井
                    for (SpaceOptimizeWellheadDTO spaceWell : wellDateMap.get(begin)) {
                        //取井口
                        if (baseWellMap.containsKey(spaceWell.getWellheadId())) {
                            baseWell = baseWellMap.get(spaceWell.getWellheadId());
                        } else {
                            continue;
                        }
                        //初始化(绿电时长,谷电时长,平电时长,高峰时长,尖峰时长)
                        BigDecimal greenPowerDuration = BigDecimal.ZERO;
                        BigDecimal lowPowerDuration = BigDecimal.ZERO;
                        BigDecimal midPowerDuration = BigDecimal.ZERO;
                        BigDecimal highPowerDuration = BigDecimal.ZERO;
                        BigDecimal peakPowerDuration = BigDecimal.ZERO;
                        BigDecimal totalDuration = BigDecimal.ZERO;
                        effectView = new SpaceOptimizeEffectView();
                        //取井运行功率
                        serviceRating = ObjectUtil.defaultIfNull(baseWell.getServiceRating(), BigDecimal.ZERO);
                        //计算权重(要去头 index从1开始)
                        weightList = this.zoningList(serviceRating, rangeToList, powerList, monthStrategyList);
                        //取间开
                        if (optimizeMap.containsKey(period.getId())) {
                            periodDurationMap = optimizeMap.get(period.getId());
                            if (periodDurationMap.containsKey(spaceWell.getWellheadId())) {
                                durationMap = periodDurationMap.get(spaceWell.getWellheadId());
                                if (durationMap.containsKey(begin)) {
                                    durationList = durationMap.get(begin);
                                    //计算时长
                                    weightIndex = 1;
                                    for (SpaceOptimizeDurationDTO duration : durationList) {
                                        optimalResult.append(duration.getOpenWellTime())
                                                .append("至")
                                                .append(duration.getCloseWellTime())
                                                .append("开井;");
                                        openTime = DateUtil.parse(duration.getOpenWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                        if (CharSequenceUtil.equals(duration.getCloseWellTime(), BusinessConstant.END_OF_DAY_TIME)) {
                                            closeTime = BusinessConstant.DATE_FLAG;
                                        } else {
                                            closeTime = DateUtil.parse(duration.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                        }
                                        for (int i = weightIndex; i < weightList.size(); i++) {
                                            weight = weightList.get(i);
                                            timestamp = weight.getTimestamp();
                                            if (DateUtil.compare(timestamp, openTime) > 0 && DateUtil.compare(timestamp, closeTime) <= 0) {
                                                switch (weight.getWeight()) {
                                                    case 0:
                                                        peakPowerDuration = peakPowerDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                                        break;
                                                    case 1:
                                                        highPowerDuration = highPowerDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                                        break;
                                                    case 3:
                                                        midPowerDuration = midPowerDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                                        break;
                                                    case 5:
                                                        lowPowerDuration = lowPowerDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                                        break;
                                                    case 10:
                                                        greenPowerDuration = greenPowerDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                                        break;
                                                    default:
                                                }
                                                totalDuration = totalDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                            } else if (DateUtil.compare(timestamp, closeTime) > 0) {
                                                weightIndex = i;
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        BaseUtils.setBaseModelDefaultForJob(effectView);
                        effectView.setIsDeleted(BusinessConstant.ONE);
                        effectView.setOptimizeDate(begin);
                        effectView.setPeriodId(period.getId());
                        effectView.setWellheadId(baseWell.getWellheadId());
                        effectView.setWellNumber(baseWell.getWellNumber());
                        effectView.setOptimalGreenPowerDuration(greenPowerDuration);
                        effectView.setOptimalLowPowerDuration(lowPowerDuration);
                        effectView.setOptimalMidPowerDuration(midPowerDuration);
                        effectView.setOptimalHighPowerDuration(highPowerDuration);
                        effectView.setOptimalPeakPowerDuration(peakPowerDuration);
                        effectView.setOptimalTotalDuration(totalDuration);
                        effectView.setOptimalResult(optimalResult.toString());
                        optimalResult.delete(0, optimalResult.length());
                        //初始化
                        greenPowerDuration = BigDecimal.ZERO;
                        lowPowerDuration = BigDecimal.ZERO;
                        midPowerDuration = BigDecimal.ZERO;
                        highPowerDuration = BigDecimal.ZERO;
                        peakPowerDuration = BigDecimal.ZERO;
                        totalDuration = BigDecimal.ZERO;
                        if (unOptimizeMap.containsKey(period.getId())) {
                            periodDurationMap = unOptimizeMap.get(period.getId());
                            if (periodDurationMap.containsKey(spaceWell.getWellheadId())) {
                                durationMap = periodDurationMap.get(spaceWell.getWellheadId());
                                if (durationMap.containsKey(begin)) {
                                    durationList = durationMap.get(begin);
                                    //计算时长
                                    weightIndex = 1;
                                    for (SpaceOptimizeDurationDTO duration : durationList) {
                                        optimalResult.append(duration.getOpenWellTime())
                                                .append("至")
                                                .append(duration.getCloseWellTime())
                                                .append("开井;");
                                        openTime = DateUtil.parse(duration.getOpenWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                        if (CharSequenceUtil.equals(duration.getCloseWellTime(), BusinessConstant.END_OF_DAY_TIME)) {
                                            closeTime = BusinessConstant.DATE_FLAG;
                                        } else {
                                            closeTime = DateUtil.parse(duration.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                        }
                                        for (int i = weightIndex; i < weightList.size(); i++) {
                                            weight = weightList.get(i);
                                            timestamp = weight.getTimestamp();
                                            if (DateUtil.compare(timestamp, openTime) > 0 && DateUtil.compare(timestamp, closeTime) <= 0) {
                                                switch (weight.getWeight()) {
                                                    case 0:
                                                        peakPowerDuration = peakPowerDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                                        break;
                                                    case 1:
                                                        highPowerDuration = highPowerDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                                        break;
                                                    case 3:
                                                        midPowerDuration = midPowerDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                                        break;
                                                    case 5:
                                                        lowPowerDuration = lowPowerDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                                        break;
                                                    case 10:
                                                        greenPowerDuration = greenPowerDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                                        break;
                                                    default:
                                                }
                                                totalDuration = totalDuration.add(BusinessConstant.BIG_DECIMAL_15);
                                            } else if (DateUtil.compare(timestamp, closeTime) > 0) {
                                                weightIndex = i;
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        effectView.setGreenPowerDuration(greenPowerDuration);
                        effectView.setLowPowerDuration(lowPowerDuration);
                        effectView.setMidPowerDuration(midPowerDuration);
                        effectView.setHighPowerDuration(highPowerDuration);
                        effectView.setPeakPowerDuration(peakPowerDuration);
                        effectView.setTotalDuration(totalDuration);
                        effectView.setInstitutionResult(optimalResult.toString());
                        optimalResult.delete(0, optimalResult.length());
                        //计算绿电占比
                        effectView.setGreenPowerRate(BaseUtils.getRate(effectView.getGreenPowerDuration(), effectView.getTotalDuration()));
                        effectView.setOptimalGreenPowerRate(BaseUtils.getRate(effectView.getOptimalGreenPowerDuration(), effectView.getOptimalTotalDuration()));
                        effectView.setGreenPowerUpgrade(effectView.getOptimalGreenPowerRate().subtract(effectView.getGreenPowerRate()));
                        //用能成本
                        BigDecimal powerCost = BigDecimal.ZERO;
                        BigDecimal optimalPowerCost = BigDecimal.ZERO;
                        //谷电电费
                        if (electrovalenceMap.containsKey(BusinessConstant.PERIOD_TYPE_LOW)) {
                            if (BaseUtils.greaterThan_0(effectView.getLowPowerDuration())) {
                                effectView.setLowPowerCost(BaseUtils.energyCost(electrovalenceMap.get(BusinessConstant.PERIOD_TYPE_LOW), baseWell.getServiceRating(), BaseUtils.divide(effectView.getLowPowerDuration(), BusinessConstant.BIG_DECIMAL_60)));
                                powerCost = powerCost.add(effectView.getLowPowerCost());
                            } else {
                                effectView.setLowPowerCost(BigDecimal.ZERO);
                            }
                            if (BaseUtils.greaterThan_0(effectView.getOptimalLowPowerDuration())) {
                                effectView.setOptimalLowPowerCost(BaseUtils.energyCost(electrovalenceMap.get(BusinessConstant.PERIOD_TYPE_LOW), baseWell.getServiceRating(), BaseUtils.divide(effectView.getOptimalLowPowerDuration(), BusinessConstant.BIG_DECIMAL_60)));
                                optimalPowerCost = optimalPowerCost.add(effectView.getOptimalLowPowerCost());
                            } else {
                                effectView.setOptimalLowPowerCost(BigDecimal.ZERO);
                            }
                        }
                        //平电电费
                        if (electrovalenceMap.containsKey(BusinessConstant.PERIOD_TYPE_PEACE)) {
                            if (BaseUtils.greaterThan_0(effectView.getMidPowerDuration())) {
                                effectView.setMidPowerCost(BaseUtils.energyCost(electrovalenceMap.get(BusinessConstant.PERIOD_TYPE_PEACE), baseWell.getServiceRating(), BaseUtils.divide(effectView.getMidPowerDuration(), BusinessConstant.BIG_DECIMAL_60)));
                                powerCost = powerCost.add(effectView.getMidPowerCost());
                            } else {
                                effectView.setMidPowerCost(BigDecimal.ZERO);
                            }
                            if (BaseUtils.greaterThan_0(effectView.getOptimalMidPowerDuration())) {
                                effectView.setOptimalMidPowerCost(BaseUtils.energyCost(electrovalenceMap.get(BusinessConstant.PERIOD_TYPE_PEACE), baseWell.getServiceRating(), BaseUtils.divide(effectView.getOptimalMidPowerDuration(), BusinessConstant.BIG_DECIMAL_60)));
                                optimalPowerCost = optimalPowerCost.add(effectView.getOptimalMidPowerCost());
                            } else {
                                effectView.setOptimalMidPowerCost(BigDecimal.ZERO);
                            }
                        }
                        //高峰电费
                        if (electrovalenceMap.containsKey(BusinessConstant.PERIOD_TYPE_PEAK)) {
                            if (BaseUtils.greaterThan_0(effectView.getHighPowerDuration())) {
                                effectView.setHighPowerCost(BaseUtils.energyCost(electrovalenceMap.get(BusinessConstant.PERIOD_TYPE_PEAK), baseWell.getServiceRating(), BaseUtils.divide(effectView.getHighPowerDuration(), BusinessConstant.BIG_DECIMAL_60)));
                                powerCost = powerCost.add(effectView.getHighPowerCost());
                            } else {
                                effectView.setHighPowerCost(BigDecimal.ZERO);
                            }
                            if (BaseUtils.greaterThan_0(effectView.getOptimalHighPowerDuration())) {
                                effectView.setOptimalHighPowerCost(BaseUtils.energyCost(electrovalenceMap.get(BusinessConstant.PERIOD_TYPE_PEAK), baseWell.getServiceRating(), BaseUtils.divide(effectView.getOptimalHighPowerDuration(), BusinessConstant.BIG_DECIMAL_60)));
                                optimalPowerCost = optimalPowerCost.add(effectView.getOptimalHighPowerCost());
                            } else {
                                effectView.setOptimalHighPowerCost(BigDecimal.ZERO);
                            }
                        }
                        //尖峰电费
                        if (electrovalenceMap.containsKey(BusinessConstant.PERIOD_TYPE_RUSH)) {
                            if (BaseUtils.greaterThan_0(effectView.getPeakPowerDuration())) {
                                effectView.setPeakPowerCost(BaseUtils.energyCost(electrovalenceMap.get(BusinessConstant.PERIOD_TYPE_RUSH), baseWell.getServiceRating(), BaseUtils.divide(effectView.getPeakPowerDuration(), BusinessConstant.BIG_DECIMAL_60)));
                                powerCost = powerCost.add(effectView.getPeakPowerCost());
                            } else {
                                effectView.setPeakPowerCost(BigDecimal.ZERO);
                            }
                            if (BaseUtils.greaterThan_0(effectView.getOptimalPeakPowerDuration())) {
                                effectView.setOptimalPeakPowerCost(BaseUtils.energyCost(electrovalenceMap.get(BusinessConstant.PERIOD_TYPE_RUSH), baseWell.getServiceRating(), BaseUtils.divide(effectView.getOptimalPeakPowerDuration(), BusinessConstant.BIG_DECIMAL_60)));
                                optimalPowerCost = optimalPowerCost.add(effectView.getOptimalPeakPowerCost());
                            } else {
                                effectView.setOptimalPeakPowerCost(BigDecimal.ZERO);
                            }
                        }
                        //绿电电费
                        if (BaseUtils.greaterThan_0(effectView.getGreenPowerDuration())) {
                            effectView.setGreenPowerCost(BaseUtils.energyCost(BusinessConstant.BIG_DECIMAL_0_3587, baseWell.getServiceRating(), BaseUtils.divide(effectView.getGreenPowerDuration(), BusinessConstant.BIG_DECIMAL_60)));
                            powerCost = powerCost.add(effectView.getGreenPowerCost());
                        } else {
                            effectView.setGreenPowerCost(BigDecimal.ZERO);
                        }
                        if (BaseUtils.greaterThan_0(effectView.getOptimalGreenPowerDuration())) {
                            effectView.setOptimalGreenPowerCost(BaseUtils.energyCost(BusinessConstant.BIG_DECIMAL_0_3587, baseWell.getServiceRating(), BaseUtils.divide(effectView.getOptimalGreenPowerDuration(), BusinessConstant.BIG_DECIMAL_60)));
                            optimalPowerCost = optimalPowerCost.add(effectView.getOptimalGreenPowerCost());
                        } else {
                            effectView.setOptimalGreenPowerCost(BigDecimal.ZERO);
                        }
                        effectView.setPowerCost(powerCost);
                        effectView.setOptimalPowerCost(optimalPowerCost);
                        //降低费用:优化前-优化后
                        effectView.setCostDown(effectView.getPowerCost().subtract(effectView.getOptimalPowerCost()));
                        effectList.add(effectView);
                    }
                }
            }
            //入库
            XTransactionHelper.begin(context, () -> {
                if (CollUtil.isNotEmpty(effectList)) {
                    SpaceOptimizeEffectViewMapper mapper = context.getBean(SpaceOptimizeEffectViewMapper.class);
                    if (effectList.size() > BaseUtils.BATCH_SIZE) {
                        List<List<SpaceOptimizeEffectView>> subList = BaseUtils.getSubList(effectList);
                        subList.forEach(mapper::batchInsert);
                    } else {
                        mapper.batchInsert(effectList);
                    }
                }
                return XServiceResult.OK;
            });
        }
    }

    /**
     * 获取优化详情
     *
     * @param context  上下文
     * @param input    输入
     * @param flag     旗帜
     * @param dateType 日期类型
     * @return {@link GetSpaceOptimizePeriodOutput}
     */
    public GetSpaceOptimizePeriodOutput getSpaceOptimizePeriodView(XContext context, GetSpaceOptimizePeriodInput input, Integer flag, Integer dateType) {
        String periodId = input.getId();
        //查制度详情
        GetSpaceOptimizePeriodOutput output = this.getOptimizePeriodDetail(context, flag, periodId);
        Date optimizeDeadline = ObjectUtil.defaultIfNull(output.getOptimizeDeadline(), output.getOptimizeDate());
        //设置优化日期列表
        output.setDateList(ServiceUtil.getOptimizeDateList(output.getCreateTime(), optimizeDeadline));
        //查间开制度详情
        SpaceInstitutionDetailEnt detailEnt = this.getInstitutionDetailById(context, output.getInstitutionId());
        //设置电网类型
        output.setGridTypeKey(detailEnt.getGridTypeKey());
        DateTime optimizeDate;
        if (BusinessConstant.FOUR.equals(flag)) {
            //一日优化时间处理
            optimizeDate = DateUtil.beginOfDay(optimizeDeadline);
        } else {
            optimizeDate = DateUtil.beginOfDay(ObjectUtil.defaultIfNull(input.getOptimizeDate(), output.getCreateTime()));
        }
        //设置市电峰谷策略
        if (CharSequenceUtil.isNotBlank(detailEnt.getGridTypeKey()) && CharSequenceUtil.equals("1", detailEnt.getGridTypeKey())) {
            output.setStrategyDetailList(ServiceUtil.getStrategyDetailList(context,
                    GetBasePriceStrategyDetailInput.builder()
                            .lineId(output.getLineId())
                            .strategyMonth(String.valueOf(optimizeDate.monthBaseOne()))
                            .build()));
        } else {
            output.setStrategyDetailList(Collections.emptyList());
        }
        //查功率预测信息
        List<DynamicQueryPlantPredictedPowerOutput> powerList = this.getAveragePowerGenerationList(context, DynamicQueryPlantPredictedPowerInput.builder()
                .plantIds(this.getOptimizeLineRelation(context, CollUtil.newArrayList(detailEnt))
                        .getPlantIdsByLineIdMap()
                        .get(detailEnt.getLineId()))
                .startTime(optimizeDate.toString())
                .endTime(DateUtil.offsetDay(optimizeDate, 1).toString())
                .dateType(dateType)
                .formatType(1)
                .build());
        Map<String, BigDecimal> collect;
        if (CollUtil.isNotEmpty(powerList)) {
            collect = powerList.stream()
                    .collect(Collectors.toMap(DynamicQueryPlantPredictedPowerOutput::getHourTime, DynamicQueryPlantPredictedPowerOutput::getPower));
        } else {
            collect = Collections.emptyMap();
        }
        List<GetPredictedPowerOutput> powerGenerationList = new ArrayList<>(24);
        String key;
        BigDecimal power;
        for (int i = 0; i < 24; i++) {
            key = String.valueOf(i);
            power = collect.getOrDefault(key, BigDecimal.ZERO);
            powerGenerationList.add(GetPredictedPowerOutput.builder()
                    .hourTime(key)
                    .power(power)
                    .build());
        }
        output.setPowerGenerationList(powerGenerationList);
        //查井口列表
        output.setWellheadList(this.getSpaceOptimizeWellheadList(context, flag, periodId, optimizeDate, powerList));
        return output;
    }

    /**
     * 获得生效中的基础间开
     *
     * @param context 上下文
     * @param sameDay 同一天
     * @return {@link List}<{@link SpaceInstitutionDetailEnt}>
     */
    public List<SpaceInstitutionDetailEnt> getEffectiveSpaceInstitution(XContext context, Date sameDay) {
        //取生效中的基础间开
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        List<SpaceInstitutionDetailEnt> detailEntList = detailMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceInstitutionDetailEnt::getIsCurrentBasic, BusinessConstant.ZERO)
                .le(SpaceInstitutionDetailEnt::getInstitutionStartDate, sameDay)
                .ge(SpaceInstitutionDetailEnt::getInstitutionEndDate, sameDay)
        );
        if (CollUtil.isEmpty(detailEntList)) {
            //未发现可优化基础间开制度
            throw new XServiceException(BusinessError.DidNotFindSpace);
        }
        return detailEntList;
    }

    /**
     * 获取所有生效中制度关联的井口及光伏电站
     *
     * @param context       上下文
     * @param detailEntList 详细ent列表
     * @return {@link SpaceOptimizeLineRelation}
     */
    public SpaceOptimizeLineRelation getOptimizeLineRelation(XContext context, List<SpaceInstitutionDetailEnt> detailEntList) {
        Set<String> lineIds = new HashSet<>(32);
        Set<String> institutionIds = new HashSet<>(32);
        for (SpaceInstitutionDetailEnt spaceInstitutionDetailEnt : detailEntList) {
            lineIds.add(spaceInstitutionDetailEnt.getLineId());
            institutionIds.add(spaceInstitutionDetailEnt.getId());
        }
        //取生效中间开的所有井口
        SpaceInstitutionWellheadViewMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
        List<SpaceInstitutionWellheadView> spaceWellheadList = wellheadMapper.selectListByInstitutionIds(institutionIds);
        if (CollUtil.isEmpty(spaceWellheadList)) {
            //未发现可优化井口
            throw new XServiceException(BusinessError.DidNotFindWellhead);
        }
        this.setServiceRatingForSpaceWellheadList(context, spaceWellheadList);
        return SpaceOptimizeLineRelation.builder()
                .wellheadsByInstitutionIdIdMap(spaceWellheadList.stream()
                        .collect(Collectors.groupingBy(SpaceInstitutionWellheadView::getInstitutionId)))
                .plantIdsByLineIdMap(
                        //取线路关联的所有光伏电站ID
                        this.getPowerLinePlantListByLineIds(context, lineIds).stream()
                                .collect(
                                        Collectors.groupingBy(DynamicQueryBasePowerLinePlantOutput::getLineId,
                                                Collectors.mapping(DynamicQueryBasePowerLinePlantOutput::getPlantId, Collectors.toList()))
                                )
                )
                //取线路关联所有储能电站计算平均功率
                .storageAvgMap(this.getPowerLineStorageAvgByLineIds(context, lineIds))
                .storageIdsByLineIdMap(this.getPowerLineStorageListByLineIds(context, lineIds))
                .build();
    }

    /**
     * 并网优化
     *
     * @param context                上下文
     * @param wellheadDTOList        井口dtolist
     * @param durationDTOList        持续时间dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param spaceWellheadList      空间井口清单
     * @param detail                 细节
     * @param monthNum               月份
     * @param periodId               期间id
     * @param plantPowerInput        电厂功率输入
     * @param optimizeDate           优化日期
     */
    public void gridConnectedOptimization(XContext context, List<SpaceOptimizeWellheadDTO> wellheadDTOList,
                                          List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                          List<SpaceInstitutionWellheadView> spaceWellheadList,
                                          SpaceInstitutionDetailEnt detail, int monthNum, String periodId,
                                          DynamicQueryPlantPredictedPowerInput plantPowerInput, DateTime optimizeDate) {
        //获取当前制度对应的光伏预测数据列表
        Map<String, List<SpaceInstitutionDurationEnt>> durationMap = this.getDurationMap(context, detail.getId());
        List<DynamicQueryPlantPredictedPowerOutput> avgPowerList = this.getAveragePowerGenerationList(context, plantPowerInput);
        if (CollUtil.isEmpty(avgPowerList)) {
            //没有预测数据都不优化
            this.setUnOptimizeWellheadConfig(durationDTOList, unOptimizeDurationList, wellheadDTOList,
                    spaceWellheadList, durationMap, periodId, optimizeDate);
            return;
        }
        //取光伏出力峰值
        BigDecimal powerMax = this.getPowerMax(avgPowerList);
        //取当前制度下井口的总功率
        BigDecimal wellheadTotalPower = this.getWellheadTotalPower(spaceWellheadList, detail.getId());
        //根据类型过滤井口:大间开,连抽井不优化
        Map<Boolean, List<SpaceInstitutionWellheadView>> wellheadViewMap = this.getWellheadViewList(spaceWellheadList, detail.getId());
        //保存不需要优化的井口
        this.setUnOptimizeWellheadConfig(durationDTOList, unOptimizeDurationList, wellheadDTOList,
                wellheadViewMap.get(false), durationMap, periodId, optimizeDate);
        List<SpaceInstitutionWellheadView> wellheadViewList = wellheadViewMap.get(true);
        if (CollUtil.isEmpty(wellheadViewList)) {
            return;
        }
        if (CollUtil.isEmpty(durationMap)) {
            //没有设置时间段,无法优化
            return;
        }
        //光伏出力峰值大于等于井口总功率:绿电消纳优先策略
        if (powerMax.compareTo(wellheadTotalPower) >= BusinessConstant.ZERO) {
            //---------------------------------绿电消纳策略---------------------------------
            this.greenElectricityConsumptionStrategy(wellheadDTOList, unOptimizeDurationList, durationDTOList,
                    wellheadViewList, avgPowerList, durationMap, detail, periodId, optimizeDate);
        } else {
            //遍历井口,按发电功率大于等于光伏出力峰值条件分组
            Map<Boolean, List<SpaceInstitutionWellheadView>> collect = this.getWellheadViewMapByPower(wellheadViewList, powerMax);
            List<SpaceInstitutionWellheadView> lowWellheadList = collect.get(false);
            int size = spaceWellheadList.size();
            int lowWellheadListSize = lowWellheadList.size();
            //光伏出力峰值<任何一口井的运行功率:消峰平谷策略
            if (size == lowWellheadListSize) {
                //---------------------------------消峰平谷策略---------------------------------
                this.peakEliminationAndValleyLevelingStrategy(context, wellheadDTOList, unOptimizeDurationList, durationDTOList, wellheadViewList,
                        durationMap, detail, monthNum, periodId, optimizeDate);
            }
            //光伏出力峰值>=线路哪部分井口运行功率:满足的部分井口采用绿电消纳优先,不满足的井采用消峰平谷
            else {
                List<SpaceInstitutionWellheadView> highWellheadList = collect.get(true);
                if (CollUtil.isNotEmpty(highWellheadList)) {
                    this.greenElectricityConsumptionStrategy(wellheadDTOList, unOptimizeDurationList, durationDTOList,
                            highWellheadList, avgPowerList, durationMap, detail, periodId, optimizeDate);
                }
                this.peakEliminationAndValleyLevelingStrategy(context, wellheadDTOList, unOptimizeDurationList, durationDTOList,
                        lowWellheadList, durationMap, detail, monthNum, periodId, optimizeDate);
            }
        }
    }

    /**
     * 离网优化
     *
     * @param context                上下文
     * @param wellheadDTOList        井口dtolist
     * @param durationDTOList        持续时间dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param spaceWellheadList      空间井口清单
     * @param storageAvgMap          存储平均值映射
     * @param detail                 细节
     * @param periodId               期间id
     * @param plantPowerInput        电厂功率输入
     * @param optimizeDate           优化日期
     */
    public void offGridOptimization(XContext context, List<SpaceOptimizeWellheadDTO> wellheadDTOList, List<SpaceOptimizeDurationDTO> durationDTOList,
                                    List<SpaceOptimizeDurationDTO> unOptimizeDurationList, List<SpaceInstitutionWellheadView> spaceWellheadList,
                                    Map<String, List<DynamicQueryBasePowerLineStorageViewOutput>> storageAvgMap, SpaceInstitutionDetailEnt detail, String periodId,
                                    DynamicQueryPlantPredictedPowerInput plantPowerInput, DateTime optimizeDate) {
        //获取井口间开时间段
        Map<String, List<SpaceInstitutionDurationEnt>> durationMap = this.getDurationMap(context, detail.getId());
        //获取当前制度对应的光伏预测数据列表
        List<DynamicQueryPlantPredictedPowerOutput> avgPowerList = this.getAveragePowerGenerationList(context, plantPowerInput);
        if (CollUtil.isEmpty(avgPowerList)) {
            //没有预测数据都不优化
            this.setUnOptimizeWellheadConfig(durationDTOList, unOptimizeDurationList, wellheadDTOList,
                    spaceWellheadList, durationMap, periodId, optimizeDate);
            return;
        }
        //根据类型过滤井口:大间开,连抽井不优化
        Map<Boolean, List<SpaceInstitutionWellheadView>> wellheadViewMap = this.getWellheadViewList(spaceWellheadList, detail.getId());
        //保存不需要优化的井口
        this.setUnOptimizeWellheadConfig(durationDTOList, unOptimizeDurationList, wellheadDTOList,
                wellheadViewMap.get(false), durationMap, periodId, optimizeDate);
        List<SpaceInstitutionWellheadView> wellheadViewList = wellheadViewMap.get(true);
        if (CollUtil.isEmpty(wellheadViewList)) {
            return;
        }
        if (CollUtil.isEmpty(durationMap)) {
            //没有设置时间段,无法优化
            return;
        }
        //时间差
        int between = 0;
        //启动间隔累积
        int startInterval = 0;
        //判断是否第一口井
        boolean isFirstWellhead;
        //第一口井启动时间
        DateTime firstStartTime = null;
        //第一口井开井时间
        int firstIndex = 0;
        //井口累积运行总功率
        BigDecimal totalOperatingPower = BigDecimal.ZERO;
        //离网时间段
        List<SpaceOptimizeDurationDTO> offGridPeriodList = new ArrayList<>(32);
        for (int w = 0, wellheadSize = wellheadViewList.size(); w < wellheadSize; w++) {
            SpaceInstitutionWellheadView wellhead = wellheadViewList.get(w);
            String wellheadId = wellhead.getWellheadId();
            String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber(), wellhead.getStartSeq(), optimizeDate, BusinessConstant.ONE);
            //取当前井口最大发电量
            BigDecimal serviceRating = wellhead.getServiceRating();
            List<SpaceInstitutionDurationEnt> durationConfigList = durationMap.get(wellheadId);
            if (CollUtil.isEmpty(durationConfigList)) {
                //没有设置时间段,无法优化
                continue;
            }
            //保存原始记录
            for (SpaceInstitutionDurationEnt durationEnt : durationConfigList) {
                this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, optimizeDate);
            }
            totalOperatingPower = totalOperatingPower.add(serviceRating);
            if (w == 0) {
                //第一个井口
                isFirstWellhead = true;
            } else {
                //累加
                startInterval = startInterval + detail.getStartInterval();
                isFirstWellhead = false;
            }
            for (int d = 0, durationSize = durationConfigList.size(); d < durationSize; d++) {
                SpaceInstitutionDurationEnt duration = durationConfigList.get(d);
                DateTime startTime = DateUtil.date(duration.getOpenTime());
                Date closeTime = duration.getCloseTime();
                DateTime endTime;
                if (ObjectUtil.isNull(closeTime)) {
                    endTime = DateUtil.beginOfDay(DateUtil.tomorrow());
                } else {
                    endTime = DateUtil.date(closeTime);
                }
                //第一次启动
                if (d == 0) {
                    //第一口井的启动时间
                    if (isFirstWellhead) {
                        //计算开井时间
                        for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                            DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                            //当日时间段平均光伏出力>=第一口井运行负荷时,该时间为第一口井运行时间
                            if (predictedPower.getPower().compareTo(serviceRating) >= BusinessConstant.ZERO) {
                                firstIndex = a;
                                break;
                            }
                        }
                        DynamicQueryPlantPredictedPowerOutput start = avgPowerList.get(firstIndex);
                        DateTime startTimeOptimize;
                        DateTime endTimeOptimize;
                        //判断发电量是否为正数
                        if (start.getPower().compareTo(BigDecimal.ZERO) > 0) {
                            startTimeOptimize = DateUtil.date(start.getCreateTime());
                            firstStartTime = startTimeOptimize;
                            //计算第一次关井时间,按照间开时间段顺延
                            int startDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                            endTimeOptimize = startTimeOptimize.offsetNew(DateField.MINUTE, startDuration);
                        } else {
                            startTimeOptimize = startTime;
                            firstStartTime = startTime;
                            endTimeOptimize = endTime;
                        }
                        this.createDurationOffGridPeriod(offGridPeriodList, wellheadId, startTimeOptimize, endTimeOptimize, BusinessConstant.ZERO);
                        //计算时间偏移,取时间间隔(分钟)
                        between = BaseUtils.getTimeDifferenceMinute(startTimeOptimize, startTime);
                    }
                    //其它井口的第一次启动时间
                    else {
                        DateTime startTimeOffset = DateUtil.offsetMinute(firstStartTime, startInterval);
                        int startIndex = -1;
                        for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                            DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                            if (DateUtil.date(predictedPower.getCreateTime()).compareTo(startTimeOffset) >= 0) {
                                //判断第一口井启动时间+启动间隔时日平均光伏出力-前两口井的运行功率是否为正数
                                if (predictedPower.getPower().compareTo(totalOperatingPower) >= 0) {
                                    //确定第二口井第一次开井时间为第一口井启动时间+启动间隔
                                    startIndex = a;
                                    break;
                                }
                            }
                        }
                        DateTime startTimeOptimize;
                        DateTime endTimeOptimize;
                        //计算未优化启动间隔
                        int openDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                        if (startIndex > -1) {
                            DynamicQueryPlantPredictedPowerOutput start = avgPowerList.get(startIndex);
                            startTimeOptimize = DateUtil.date(start.getCreateTime());
                            endTimeOptimize = startTimeOptimize.offsetNew(DateField.MINUTE, openDuration);
                        } else {
                            //无法优化
                            endTimeOptimize = startTimeOffset.offsetNew(DateField.MINUTE, openDuration);
                            startTimeOptimize = startTimeOffset;
                        }
                        this.createDurationOffGridPeriod(offGridPeriodList, wellheadId, startTimeOptimize, endTimeOptimize, BusinessConstant.ZERO);
                        //取时间间隔(分钟)
                        between = BaseUtils.getTimeDifferenceMinute(endTimeOptimize, endTime);
                        //修改启动时间
                        firstStartTime = startTimeOptimize;
                    }
                } else {
                    //计算偏移
                    this.createDurationOffGridPeriod(offGridPeriodList, wellheadId,
                            DateUtil.offsetMinute(startTime, between),
                            DateUtil.offsetMinute(endTime, between), BusinessConstant.ONE
                    );
                }
            }
        }
        //离网优化
        if (CollUtil.isNotEmpty(offGridPeriodList)) {
            //拆分时间段
            for (SpaceOptimizeDurationDTO durationDTO : offGridPeriodList) {
                //-----------------------------------大庆暂时不用储能
                DateTime openTime = DateUtil.date(durationDTO.getOpenTime());
                DateTime closeTime = DateUtil.date(durationDTO.getCloseTime());
                if (openTime.compareTo(BusinessConstant.DATE_FLAG) < 0 && closeTime.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                    this.createOptimizeDuration(durationDTOList, periodId, durationDTO.getRecordId(), durationDTO.getWellheadId(), BusinessConstant.PHOTOVOLTAIC,
                            openTime.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                            durationDTO.getFirstStart(), optimizeDate
                    );
                    this.createOptimizeDuration(durationDTOList, periodId, durationDTO.getRecordId(), durationDTO.getWellheadId(), BusinessConstant.PHOTOVOLTAIC,
                            BusinessConstant.START_OF_DAY_TIME, closeTime.toString(BusinessConstant.MINUTES_FORMAT),
                            durationDTO.getFirstStart(), optimizeDate
                    );
                } else if (closeTime.compareTo(BusinessConstant.DATE_FLAG) == 0) {
                    this.createOptimizeDuration(durationDTOList, periodId, durationDTO.getRecordId(), durationDTO.getWellheadId(), BusinessConstant.PHOTOVOLTAIC,
                            openTime.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                            durationDTO.getFirstStart(), optimizeDate
                    );
                } else {
                    this.createOptimizeDuration(durationDTOList, periodId, durationDTO.getRecordId(), durationDTO.getWellheadId(), BusinessConstant.PHOTOVOLTAIC,
                            openTime.toString(BusinessConstant.MINUTES_FORMAT), closeTime.toString(BusinessConstant.MINUTES_FORMAT),
                            durationDTO.getFirstStart(), optimizeDate
                    );
                }
            }
        }
    }

    /**
     * 获取每分钟发电量列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    public List<DynamicQueryPlantPredictedPowerOutput> getAveragePowerGenerationList(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        IPlantPredictedPowerCloudService cloudService = context.getBean(IPlantPredictedPowerCloudService.class);
        XListResult<DynamicQueryPlantPredictedPowerOutput> result = cloudService.queryAveragePowerGenerationListByParam(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 创建间开优化信息
     *
     * @param periodDTOList          时期数据主义者
     * @param detailId               详细信息id
     * @param lineId                 线路id
     * @param executionCycleForMonth 月执行周期
     * @param optimizeDeadline       优化截止日期
     * @return {@link String}
     */
    public String createOptimizePeriod(List<SpaceOptimizePeriodDTO> periodDTOList, String detailId,
                                       String lineId, String executionCycleForMonth, Date optimizeDeadline) {
        SpaceOptimizePeriodDTO periodDTO = new SpaceOptimizePeriodDTO();
        BaseUtils.setBaseModelDefaultForJob(periodDTO);
        periodDTO.setInstitutionId(detailId);
        periodDTO.setLineId(lineId);
        periodDTO.setExecutionCycle(executionCycleForMonth);
        periodDTO.setOptimizeState(BusinessConstant.ZERO);
        periodDTO.setOptimizeDeadline(optimizeDeadline);
        periodDTOList.add(periodDTO);
        return periodDTO.getId();
    }

    /**
     * 创建间开优化井口信息
     *
     * @param wellheadDTOList 井口dtolist
     * @param periodId        期间id
     * @param wellheadId      井口id
     * @param wellNumber      井号
     * @param startSeq        开始seq
     * @param optimizeDate    优化日期
     * @param frostproofWell  防冻井
     * @return {@link String}
     */
    public String createOptimizeWellhead(List<SpaceOptimizeWellheadDTO> wellheadDTOList, String periodId, String wellheadId,
                                         String wellNumber, Integer startSeq, DateTime optimizeDate, Integer frostproofWell) {
        SpaceOptimizeWellheadDTO wellheadDTO = new SpaceOptimizeWellheadDTO();
        BaseUtils.setBaseModelDefaultForJob(wellheadDTO);
        wellheadDTO.setPeriodId(periodId);
        wellheadDTO.setWellheadId(wellheadId);
        wellheadDTO.setWellNumber(wellNumber);
        wellheadDTO.setStartSeq(startSeq);
        wellheadDTO.setOptimizeDate(optimizeDate);
        wellheadDTO.setFrostproofWell(frostproofWell);
        wellheadDTOList.add(wellheadDTO);
        return wellheadDTO.getId();
    }

    /**
     * 创建间开原始间开区间
     *
     * @param unOptimizeDurationList 取消优化工期列表
     * @param durationEnt            持续时间ent
     * @param periodId               期间id
     * @param recordId               记录id
     * @param wellheadId             井口id
     * @param optimizeDate           优化时间
     */
    public void createUnOptimizeDuration(List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                         SpaceInstitutionDurationEnt durationEnt,
                                         String periodId, String recordId, String wellheadId, DateTime optimizeDate) {
        this.createDuration(unOptimizeDurationList, recordId, periodId, wellheadId, BusinessConstant.ONE,
                durationEnt.getOpenWellTime(), durationEnt.getCloseWellTime(), null,
                BusinessConstant.ONE, optimizeDate);
    }

    /**
     * 创建间开优化井口区间配置信息
     *
     * @param durationDTOList   持续时间dtolist
     * @param periodId          期间id
     * @param recordId          记录id
     * @param wellheadId        井口id
     * @param generationTypeKey 生成类型密钥
     * @param openWellTime      开井时间
     * @param endTimeString     结束时间字符串
     * @param firstStart        是否第一次启动
     * @param optimizeDate      优化时间
     */
    public void createOptimizeDuration(List<SpaceOptimizeDurationDTO> durationDTOList, String periodId,
                                       String recordId, String wellheadId, String generationTypeKey,
                                       String openWellTime, String endTimeString, Integer firstStart, Date optimizeDate) {
        this.createDuration(durationDTOList, recordId, periodId, wellheadId, BusinessConstant.ZERO,
                openWellTime, endTimeString, generationTypeKey, firstStart, optimizeDate);
    }

    /**
     * 创建持续时间
     *
     * @param durationDTOList   持续时间dtolist
     * @param recordId          记录id
     * @param periodId          期间id
     * @param wellheadId        井口id
     * @param isOptimize        正在优化
     * @param openWellTime      开井时间
     * @param closeWellTime     关井时间
     * @param generationTypeKey 生成类型密钥
     * @param firstStart        是否第一次启动
     * @param optimizeDate      优化时间
     */
    public void createDuration(List<SpaceOptimizeDurationDTO> durationDTOList, String recordId,
                               String periodId, String wellheadId, Integer isOptimize, String openWellTime,
                               String closeWellTime, String generationTypeKey, Integer firstStart, Date optimizeDate) {
        SpaceOptimizeDurationDTO durationDTO = new SpaceOptimizeDurationDTO();
        BaseUtils.setBaseModelDefaultForJob(durationDTO);
        durationDTO.setRecordId(recordId);
        durationDTO.setPeriodId(periodId);
        durationDTO.setWellheadId(wellheadId);
        durationDTO.setIsOptimize(isOptimize);
        durationDTO.setOpenWellTime(openWellTime);
        durationDTO.setCloseWellTime(closeWellTime);
        durationDTO.setGenerationTypeKey(generationTypeKey);
        durationDTO.setFirstStart(firstStart);
        durationDTO.setOptimizeDate(optimizeDate);
        durationDTOList.add(durationDTO);
    }

    /**
     * 获得平均功率
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    public List<DynamicQueryPlantPredictedPowerOutput> getAveragePower(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        IPlantPredictedPowerCloudService cloudService = context.getBean(IPlantPredictedPowerCloudService.class);
        XListResult<DynamicQueryPlantPredictedPowerOutput> result = cloudService.queryAveragePower(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 按id获取间开详细信息
     *
     * @param context       上下文
     * @param institutionId 机构id
     * @return {@link SpaceInstitutionDetailEnt}
     */
    private SpaceInstitutionDetailEnt getInstitutionDetailById(XContext context, String institutionId) {
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        return detailMapper.selectById(institutionId);
    }

    /**
     * 获取优化期间详细信息
     *
     * @param context  上下文
     * @param flag     旗帜
     * @param periodId 期间id
     * @return {@link GetSpaceOptimizePeriodOutput}
     */
    private GetSpaceOptimizePeriodOutput getOptimizePeriodDetail(XContext context, Integer flag, String periodId) {
        GetSpaceOptimizePeriodOutput output = null;
        switch (flag) {
            case 1:
                output = XCopyUtils.copyNewObject(context.getBean(SpaceOptimizeUltraPeriodMapper.class).selectById(periodId),
                        GetSpaceOptimizePeriodOutput.class);
                break;
            case 2:
                output = XCopyUtils.copyNewObject(context.getBean(SpaceOptimizeLongPeriodMapper.class).selectById(periodId),
                        GetSpaceOptimizePeriodOutput.class);
                break;
            case 3:
                output = XCopyUtils.copyNewObject(context.getBean(SpaceOptimizeMidPeriodMapper.class).selectById(periodId),
                        GetSpaceOptimizePeriodOutput.class);
                break;
            case 4:
                output = XCopyUtils.copyNewObject(context.getBean(SpaceOptimizeShortPeriodMapper.class).selectById(periodId),
                        GetSpaceOptimizePeriodOutput.class);
                break;
            default:
        }
        if (ObjectUtil.isNull(output)) {
            throw new XServiceException(context, XError.NotFound);
        }
        return output;
    }

    /**
     * 获取空间优化井口列表
     *
     * @param context      上下文
     * @param flag         旗帜
     * @param periodId     期间id
     * @param optimizeDate 优化日期
     * @param powerList    预测列表
     * @return {@link List}<{@link GetSpaceOptimizeWellheadOutput}>
     */
    private List<GetSpaceOptimizeWellheadOutput> getSpaceOptimizeWellheadList(XContext context, Integer flag, String periodId, DateTime optimizeDate, List<DynamicQueryPlantPredictedPowerOutput> powerList) {
        //井口列表
        List<GetSpaceOptimizeWellheadOutput> wellheadOutputs = null;
        //配置列表
        List<SpaceOptimizeDateDuration> durationOutputs = null;
        switch (flag) {
            case 1:
                wellheadOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeUltraWellheadViewMapper.class)
                        .selectListJoin(SpaceOptimizeUltraWellheadView.builder()
                                .ultraPeriodId(periodId)
                                .optimizeDate(optimizeDate)
                                .build()), GetSpaceOptimizeWellheadOutput.class);
                durationOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeUltraDurationMapper.class)
                        .selectList(new LambdaQueryWrapper<SpaceOptimizeUltraDurationEnt>()
                                .eq(SpaceOptimizeUltraDurationEnt::getUltraPeriodId, periodId)
                                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                                .eq(SpaceOptimizeUltraDurationEnt::getOptimizeDate, optimizeDate)
                                .orderByAsc(SpaceOptimizeUltraDurationEnt::getIsOptimize, SpaceOptimizeUltraDurationEnt::getOpenWellTime)
                        ), SpaceOptimizeDateDuration.class);
                break;
            case 2:
                wellheadOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeLongWellheadViewMapper.class)
                        .selectListJoin(SpaceOptimizeLongWellheadView.builder()
                                .longPeriodId(periodId)
                                .optimizeDate(optimizeDate)
                                .build()), GetSpaceOptimizeWellheadOutput.class);
                durationOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeLongDurationMapper.class)
                        .selectList(new LambdaQueryWrapper<SpaceOptimizeLongDurationEnt>()
                                .eq(SpaceOptimizeLongDurationEnt::getLongPeriodId, periodId)
                                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                                .eq(SpaceOptimizeLongDurationEnt::getOptimizeDate, optimizeDate)
                                .orderByAsc(SpaceOptimizeLongDurationEnt::getIsOptimize, SpaceOptimizeLongDurationEnt::getOpenWellTime)
                        ), SpaceOptimizeDateDuration.class);
                break;
            case 3:
                wellheadOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeMidWellheadViewMapper.class)
                        .selectListJoin(SpaceOptimizeMidWellheadView.builder()
                                .midPeriodId(periodId)
                                .optimizeDate(optimizeDate)
                                .build()), GetSpaceOptimizeWellheadOutput.class);
                durationOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeMidDurationMapper.class)
                        .selectList(new LambdaQueryWrapper<SpaceOptimizeMidDurationEnt>()
                                .eq(SpaceOptimizeMidDurationEnt::getMidPeriodId, periodId)
                                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                                .eq(SpaceOptimizeMidDurationEnt::getOptimizeDate, optimizeDate)
                                .orderByAsc(SpaceOptimizeMidDurationEnt::getIsOptimize, SpaceOptimizeMidDurationEnt::getOpenWellTime)
                        ), SpaceOptimizeDateDuration.class);
                break;
            case 4:
                wellheadOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeShortWellheadViewMapper.class)
                        .selectListJoin(SpaceOptimizeShortWellheadView.builder()
                                .shortPeriodId(periodId)
                                .optimizeDate(optimizeDate)
                                .build()), GetSpaceOptimizeWellheadOutput.class);
                durationOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeShortDurationMapper.class)
                        .selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                                .eq(SpaceOptimizeShortDurationEnt::getShortPeriodId, periodId)
                                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                                .eq(SpaceOptimizeShortDurationEnt::getOptimizeDate, optimizeDate)
                                .orderByAsc(SpaceOptimizeShortDurationEnt::getIsOptimize, SpaceOptimizeShortDurationEnt::getOpenWellTime)
                        ), SpaceOptimizeDateDuration.class);
                break;
            default:
        }
        if (CollUtil.isEmpty(wellheadOutputs)) {
            wellheadOutputs = Collections.emptyList();
        } else {
            if (CollUtil.isNotEmpty(durationOutputs)) {
                //查能耗
                SpaceOptimizeEffectMapper mapper = context.getBean(SpaceOptimizeEffectMapper.class);
                List<SpaceOptimizeEffectEnt> effectList = mapper.selectList(new LambdaQueryWrapper<SpaceOptimizeEffectEnt>()
                        .select(SpaceOptimizeEffectEnt::getWellheadId, SpaceOptimizeEffectEnt::getGreenPowerRate, SpaceOptimizeEffectEnt::getPowerCost,
                                SpaceOptimizeEffectEnt::getOptimalGreenPowerRate, SpaceOptimizeEffectEnt::getOptimalPowerCost, SpaceOptimizeEffectEnt::getInstitutionResult,
                                SpaceOptimizeEffectEnt::getGreenPowerUpgrade, SpaceOptimizeEffectEnt::getCostDown, SpaceOptimizeEffectEnt::getOptimalResult)
                        .eq(BaseModel::getIsDeleted, 1)
                        .eq(SpaceOptimizeEffectEnt::getPeriodId, periodId)
                        .eq(SpaceOptimizeEffectEnt::getOptimizeDate, optimizeDate)
                );
                Map<String, SpaceOptimizeEffectEnt> effectMap;
                if (CollUtil.isNotEmpty(effectList)) {
                    effectMap = effectList.stream()
                            .collect(Collectors.toMap(SpaceOptimizeEffectEnt::getWellheadId, Function.identity()));
                } else {
                    effectMap = Collections.emptyMap();
                }
                //处理间开
                Map<String, List<SpaceOptimizeDateDuration>> collect = new HashMap<>(16);
                for (SpaceOptimizeDateDuration durationOutput : durationOutputs) {
                    if (CharSequenceUtil.isEmpty(durationOutput.getGenerationTypeKey())) {
                        durationOutput.setGenerationTypeKey(BusinessConstant.PHOTOVOLTAIC);
                    }
                    List<SpaceOptimizeDateDuration> tmpList = collect.get(durationOutput.getWellheadId());
                    if (CollUtil.isEmpty(tmpList)) {
                        tmpList = new ArrayList<>(16);
                        tmpList.add(durationOutput);
                        collect.put(durationOutput.getWellheadId(), tmpList);
                    } else {
                        tmpList.add(durationOutput);
                    }
                }
                //封装
                Map<Integer, List<SpaceOptimizeDateDuration>> listMap;
                SpaceOptimizeEffectEnt effect;
                boolean isOptimal;
                BigDecimal costDown;
                String optimalResult;
                BigDecimal optimalGreenPowerRate;
                BigDecimal optimalPowerCost;
                BigDecimal greenPowerUpgrade;
                for (GetSpaceOptimizeWellheadOutput w : wellheadOutputs) {
                    //封装能耗分析
                    if (effectMap.containsKey(w.getWellheadId())) {
                        effect = effectMap.get(w.getWellheadId());
                        greenPowerUpgrade = effect.getGreenPowerUpgrade();
                        //绿电占比大于等于0才优化
                        isOptimal = greenPowerUpgrade.compareTo(BigDecimal.ZERO) >= 0;
                        if (isOptimal) {
                            optimalResult = effect.getOptimalResult();
                            optimalGreenPowerRate = effect.getOptimalGreenPowerRate();
                            optimalPowerCost = effect.getOptimalPowerCost();
                            costDown = effect.getCostDown();
                        } else {
                            optimalResult = effect.getInstitutionResult();
                            optimalGreenPowerRate = effect.getGreenPowerRate();
                            optimalPowerCost = effect.getPowerCost();
                            greenPowerUpgrade = BigDecimal.ZERO;
                            costDown = BigDecimal.ZERO;
                        }
                        w.setEffect(GetSpaceOptimizeEffectOutput.builder()
                                .optimalResult(optimalResult)
                                .greenPowerRate(effect.getGreenPowerRate())
                                .powerCost(effect.getPowerCost())
                                .optimalGreenPowerRate(optimalGreenPowerRate)
                                .optimalPowerCost(optimalPowerCost)
                                .greenPowerUpgrade(greenPowerUpgrade)
                                .costDown(costDown)
                                .build());
                    } else {
                        w.setEffect(new GetSpaceOptimizeEffectOutput());
                        isOptimal = false;
                    }
                    //封装间开
                    if (collect.containsKey(w.getWellheadId())) {
                        listMap = collect.get(w.getWellheadId()).stream()
                                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getIsOptimize));
                        //填充关井时间(降低成本大于等于0才优化)
                        if (isOptimal) {
                            w.setOptimizeList(ServiceUtil.getAllDateDuration(listMap.get(0)));
                        } else {
                            w.setOptimizeList(ServiceUtil.getAllDateDuration(listMap.get(1)));
                        }
                        w.setInitializeList(ServiceUtil.getAllDateDuration(listMap.get(1)));
                    } else {
                        w.setOptimizeList(Collections.emptyList());
                        w.setInitializeList(Collections.emptyList());
                    }
                }
            }
        }
        return wellheadOutputs;
    }

    /**
     * 绿电消纳策略
     *
     * @param wellheadDTOList        井口dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param durationDTOList        持续时间dtolist
     * @param wellheadViewList       井口视图列表
     * @param avgPowerList           平均功率列表
     * @param durationMap            持续时间图
     * @param detail                 细节
     * @param periodId               期间id
     * @param optimizeDate           优化日期
     */
    private void greenElectricityConsumptionStrategy(List<SpaceOptimizeWellheadDTO> wellheadDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                                     List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceInstitutionWellheadView> wellheadViewList,
                                                     List<DynamicQueryPlantPredictedPowerOutput> avgPowerList, Map<String, List<SpaceInstitutionDurationEnt>> durationMap,
                                                     SpaceInstitutionDetailEnt detail, String periodId, DateTime optimizeDate) {
        //时间差
        int between = 0;
        //启动间隔累积
        int startInterval = 0;
        //判断是否第一口井
        boolean isFirstWellhead;
        //第一口井启动时间
        DateTime firstStartTime = null;
        //井口累积运行总功率
        BigDecimal totalOperatingPower = BigDecimal.ZERO;
        //临时间开
        List<SpaceOptimizeDurationDTO> newDurationDTOList;
        for (int w = 0, wellheadSize = wellheadViewList.size(); w < wellheadSize; w++) {
            newDurationDTOList = new ArrayList<>(12);
            SpaceInstitutionWellheadView wellhead = wellheadViewList.get(w);
            String wellheadId = wellhead.getWellheadId();
            String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber(), wellhead.getStartSeq(), optimizeDate, BusinessConstant.ONE);
            //取当前井口最大发电量
            BigDecimal serviceRating = wellhead.getServiceRating();
            List<SpaceInstitutionDurationEnt> durationConfigList = durationMap.get(wellheadId);
            if (CollUtil.isEmpty(durationConfigList)) {
                //没有设置时间段,无法优化
                continue;
            }
            //保存原始记录
            for (SpaceInstitutionDurationEnt durationEnt : durationConfigList) {
                this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, optimizeDate);
            }
            //累加
            totalOperatingPower = totalOperatingPower.add(serviceRating);
            if (w == 0) {
                //第一个井口
                isFirstWellhead = true;
            } else {
                startInterval = startInterval + detail.getStartInterval();
                isFirstWellhead = false;
            }
            for (int d = 0, durationSize = durationConfigList.size(); d < durationSize; d++) {
                SpaceInstitutionDurationEnt duration = durationConfigList.get(d);
                DateTime startTime = DateUtil.date(duration.getOpenTime());
                Date closeTime = duration.getCloseTime();
                DateTime endTime;
                if (ObjectUtil.isNull(closeTime)) {
                    endTime = BusinessConstant.DATE_FLAG;
                } else {
                    endTime = DateUtil.date(closeTime);
                }
                //第一次启动
                if (d == 0) {
                    //第一口井的启动时间
                    if (isFirstWellhead) {
                        //计算开井时间
                        int startIndex = 0;
                        for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                            DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                            //当日时间段平均光伏出力>=第一口井运行负荷时,该时间为第一口井运行时间
                            if (predictedPower.getPower().compareTo(serviceRating) >= BusinessConstant.ZERO) {
                                startIndex = a;
                                break;
                            }
                        }
                        DynamicQueryPlantPredictedPowerOutput start = avgPowerList.get(startIndex);
                        DateTime startTimeOptimize = DateUtil.date(start.getCreateTime());
                        //计算第一次关井时间,按照间开时间段顺延
                        int startDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                        DateTime endTimeOptimize = startTimeOptimize.offsetNew(DateField.MINUTE, startDuration);
                        if (startTimeOptimize.compareTo(BusinessConstant.DATE_FLAG) < 0 && endTimeOptimize.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                            this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                    startTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                                    BusinessConstant.ZERO, optimizeDate
                            );
                            this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                    BusinessConstant.START_OF_DAY_TIME, endTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT),
                                    BusinessConstant.ZERO, optimizeDate
                            );
                        } else if (endTimeOptimize.compareTo(BusinessConstant.DATE_FLAG) == 0) {
                            this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                    startTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                                    BusinessConstant.ZERO, optimizeDate
                            );
                        } else {
                            this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                    startTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT), endTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT),
                                    BusinessConstant.ZERO, optimizeDate
                            );
                        }
                        firstStartTime = startTimeOptimize;
                        //计算时间偏移,取时间间隔(分钟)
                        between = BaseUtils.getTimeDifferenceMinute(startTimeOptimize, startTime);
                    }
                    //其它井口的第一次启动时间
                    else {
                        DateTime startTimeOffset = DateUtil.offsetMinute(firstStartTime, startInterval);
                        int startIndex = -1;
                        for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                            DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                            if (DateUtil.date(predictedPower.getCreateTime()).compareTo(startTimeOffset) >= 0) {
                                //判断第一口井启动时间+启动间隔时日平均光伏出力-前两口井的运行功率是否为正数
                                if (predictedPower.getPower().compareTo(totalOperatingPower) >= 0) {
                                    //确定第二口井第一次开井时间为第一口井启动时间+启动间隔
                                    startIndex = a;
                                    break;
                                }
                            }
                        }
                        DateTime startTimeOptimize;
                        DateTime endTimeOptimize;
                        if (startIndex > -1) {
                            DynamicQueryPlantPredictedPowerOutput start = avgPowerList.get(startIndex);
                            startTimeOptimize = DateUtil.date(start.getCreateTime());
                            //计算未优化启动间隔
                            int openDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                            endTimeOptimize = startTimeOptimize.offsetNew(DateField.MINUTE, openDuration);
                        } else
                        //优化不了,用第一次启动时间+启动间隔往后挪
                        {
                            //计算未优化启动间隔
                            int openDuration = (int) DateUtil.between(startTime, endTime, DateUnit.MINUTE);
                            startTimeOptimize = DateUtil.offsetMinute(firstStartTime, startInterval);
                            endTimeOptimize = startTimeOptimize.offsetNew(DateField.MINUTE, openDuration);
                        }
                        //记录
                        if (startTimeOptimize.compareTo(BusinessConstant.DATE_FLAG) < 0 && endTimeOptimize.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                            this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                    startTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                                    BusinessConstant.ZERO, optimizeDate
                            );
                            this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                    BusinessConstant.START_OF_DAY_TIME, endTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT),
                                    BusinessConstant.ZERO, optimizeDate
                            );
                        } else if (endTimeOptimize.compareTo(BusinessConstant.DATE_FLAG) == 0) {
                            this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                    startTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                                    BusinessConstant.ZERO, optimizeDate
                            );
                        } else {
                            this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                    startTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT), endTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT),
                                    BusinessConstant.ZERO, optimizeDate
                            );
                        }
                        //取时间间隔(分钟)
                        between = BaseUtils.getTimeDifferenceMinute(endTimeOptimize, endTime);
                        //修改启动时间
                        firstStartTime = startTimeOptimize;
                    }
                } else {
                    DateTime startOffset = DateUtil.offsetMinute(startTime, between);
                    DateTime endOffset = DateUtil.offsetMinute(endTime, between);
                    if (startOffset.compareTo(BusinessConstant.DATE_FLAG) < 0 && endOffset.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                                BusinessConstant.ONE, optimizeDate
                        );
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                BusinessConstant.START_OF_DAY_TIME, endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ONE, optimizeDate
                        );
                    } else if (endOffset.compareTo(BusinessConstant.DATE_FLAG) == 0) {
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                                BusinessConstant.ONE, optimizeDate
                        );
                    } else {
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT), endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ONE, optimizeDate
                        );
                    }
                }
            }
            //数据处理
            if (CollUtil.isNotEmpty(newDurationDTOList)) {
                newDurationDTOList.sort(Comparator.comparing(SpaceOptimizeDurationDTO::getOpenWellTime));
                List<SpaceOptimizeDurationDTO> tempDurationDTOList = new ArrayList<>(newDurationDTOList.size());
                tempDurationDTOList.add(newDurationDTOList.get(0));
                for (int i = 1; i < newDurationDTOList.size(); i++) {
                    SpaceOptimizeDurationDTO nowOne = newDurationDTOList.get(i);
                    SpaceOptimizeDurationDTO lastOne = tempDurationDTOList.get(tempDurationDTOList.size() - 1);
                    if (CharSequenceUtil.equals(lastOne.getCloseWellTime(), nowOne.getOpenWellTime())) {
                        lastOne.setCloseWellTime(nowOne.getCloseWellTime());
                    } else {
                        tempDurationDTOList.add(nowOne);
                    }
                }
                //保存结果
                for (SpaceOptimizeDurationDTO dto : tempDurationDTOList) {
                    durationDTOList.add(dto);
                }
            }
        }
    }

    /**
     * 消峰平谷策略
     *
     * @param context                上下文
     * @param wellheadDTOList        井口dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param durationDTOList        持续时间dtolist
     * @param wellheadViewList       井口视图列表
     * @param durationMap            持续时间图
     * @param detail                 细节
     * @param monthNum               月份
     * @param periodId               id
     * @param optimizeDate           优化日期
     */
    private void peakEliminationAndValleyLevelingStrategy(XContext context, List<SpaceOptimizeWellheadDTO> wellheadDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                                          List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceInstitutionWellheadView> wellheadViewList,
                                                          Map<String, List<SpaceInstitutionDurationEnt>> durationMap, SpaceInstitutionDetailEnt detail,
                                                          int monthNum, String periodId, DateTime optimizeDate) {
        //判断是否第一口井
        boolean isFirstWellhead;
        //时间差
        int between = 0;
        //启动间隔累积
        int startInterval = 0;
        //第一口井启动时间
        DateTime firstStartTime = null;
        //第一口井结束时间
        DateTime firstEndTime = null;
        //临时间开
        List<SpaceOptimizeDurationDTO> newDurationDTOList;
        for (int w = 0, wellheadSize = wellheadViewList.size(); w < wellheadSize; w++) {
            newDurationDTOList = new ArrayList<>(12);
            SpaceInstitutionWellheadView wellhead = wellheadViewList.get(w);
            String wellheadId = wellhead.getWellheadId();
            String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber(), wellhead.getStartSeq(), optimizeDate, BusinessConstant.ONE);
            List<SpaceInstitutionDurationEnt> durationConfigList = durationMap.get(wellheadId);
            if (CollUtil.isEmpty(durationConfigList)) {
                //没有设置时间段,无法优化
                continue;
            }
            //保存原始记录
            for (SpaceInstitutionDurationEnt durationEnt : durationConfigList) {
                this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, optimizeDate);
            }
            if (w == 0) {
                //第一个井口
                isFirstWellhead = true;
            } else {
                //计算启动间隔
                startInterval = startInterval + detail.getStartInterval();
                isFirstWellhead = false;
            }
            for (int d = 0, durationSize = durationConfigList.size(); d < durationSize; d++) {
                SpaceInstitutionDurationEnt duration = durationConfigList.get(d);
                DateTime startTime = DateUtil.date(duration.getOpenTime());
                Date closeTime = duration.getCloseTime();
                DateTime endTime;
                if (ObjectUtil.isNull(closeTime)) {
                    endTime = BusinessConstant.DATE_FLAG;
                } else {
                    endTime = DateUtil.date(closeTime);
                }
                //第一次启动
                if (d == 0) {
                    DateTime endTimeOptimize;
                    //第一口井的启动时间
                    if (isFirstWellhead) {
                        //通过线路ID和月份获取市电峰谷策略明细配置
                        List<GetBasePriceStrategyDetailOutput> strategyDetailList = ServiceUtil.getStrategyDetailList(context,
                                GetBasePriceStrategyDetailInput.builder()
                                        .lineId(detail.getLineId())
                                        .strategyMonth(String.valueOf(monthNum))
                                        .build()
                        );
                        if (CollUtil.isNotEmpty(strategyDetailList)) {
                            //获取第一段谷电阶段的开始时间为第一口井的开井时间
                            Optional<GetBasePriceStrategyDetailOutput> low = strategyDetailList.stream()
                                    .filter(s -> CharSequenceUtil.isAllNotBlank(s.getEndTime(), s.getStartTime()) && CharSequenceUtil.equals(BusinessConstant.PERIOD_TYPE_LOW, s.getPeriodTypeKey()))
                                    .findFirst();
                            if (low.isPresent()) {
                                GetBasePriceStrategyDetailOutput strategyDetailOutput = low.get();
                                //第一口井启动时间
                                firstStartTime = DateUtil.parse(strategyDetailOutput.getStartTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                firstEndTime = DateUtil.parse(strategyDetailOutput.getEndTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                            } else {
                                //没有谷电,不优化
                                firstStartTime = startTime;
                                firstEndTime = endTime;
                            }
                        } else {
                            //没有配置,不优化
                            firstStartTime = startTime;
                            firstEndTime = endTime;
                        }
                        //计算优化后的时间差优化后的第一口井开井时间-优化前第一次开井时间
                        int startDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                        //第一次关井时间按照启动时长顺延
                        endTimeOptimize = firstStartTime.offsetNew(DateField.MINUTE, startDuration);
                    }//其它井口的第一次启动时间
                    else {
                        //其它井启动时间:第一口井启动时间+启动间隔
                        DateTime startTimeOffset = DateUtil.offsetMinute(firstStartTime, startInterval);
                        endTimeOptimize = DateUtil.offsetMinute(firstEndTime, startInterval);
                        //修改启动时间
                        firstStartTime = startTimeOffset;
                    }
                    //计算优化时间段
                    if (firstStartTime.compareTo(BusinessConstant.DATE_FLAG) < 0 && endTimeOptimize.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                firstStartTime.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                                BusinessConstant.ZERO, optimizeDate
                        );
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                BusinessConstant.START_OF_DAY_TIME, endTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ZERO, optimizeDate
                        );
                    } else if (endTimeOptimize.compareTo(BusinessConstant.DATE_FLAG) == 0) {
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                firstStartTime.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                                BusinessConstant.ZERO, optimizeDate
                        );
                    } else {
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                firstStartTime.toString(BusinessConstant.MINUTES_FORMAT), endTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ZERO, optimizeDate
                        );
                    }
                    //取时间间隔(分钟)
                    between = BaseUtils.getTimeDifferenceMinute(endTimeOptimize, endTime);
                } else {
                    DateTime startOffset = DateUtil.offsetMinute(startTime, between);
                    DateTime endOffset = DateUtil.offsetMinute(endTime, between);
                    if (startOffset.compareTo(BusinessConstant.DATE_FLAG) < 0 && endOffset.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                        //如果时间超过当天,舍弃
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.END_OF_DAY_TIME,
                                BusinessConstant.ONE, optimizeDate
                        );
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                BusinessConstant.START_OF_DAY_TIME,
                                endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ONE, optimizeDate
                        );
                    } else if (endOffset.compareTo(BusinessConstant.DATE_FLAG) == 0) {
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.END_OF_DAY_TIME,
                                BusinessConstant.ONE, optimizeDate
                        );
                    } else {
                        //计算偏移
                        this.createOptimizeDuration(newDurationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ONE, optimizeDate
                        );
                    }
                }
            }
            //数据处理
            if (CollUtil.isNotEmpty(newDurationDTOList)) {
                newDurationDTOList.sort(Comparator.comparing(SpaceOptimizeDurationDTO::getOpenWellTime));
                List<SpaceOptimizeDurationDTO> tempDurationDTOList = new ArrayList<>(newDurationDTOList.size());
                tempDurationDTOList.add(newDurationDTOList.get(0));
                for (int i = 1; i < newDurationDTOList.size(); i++) {
                    SpaceOptimizeDurationDTO nowOne = newDurationDTOList.get(i);
                    SpaceOptimizeDurationDTO lastOne = tempDurationDTOList.get(tempDurationDTOList.size() - 1);
                    if (CharSequenceUtil.equals(lastOne.getCloseWellTime(), nowOne.getOpenWellTime())) {
                        lastOne.setCloseWellTime(nowOne.getCloseWellTime());
                    } else {
                        tempDurationDTOList.add(nowOne);
                    }
                }
                //保存结果
                for (SpaceOptimizeDurationDTO dto : tempDurationDTOList) {
                    durationDTOList.add(dto);
                }
            }
        }
    }

    /**
     * 保存不需要优化的井口
     *
     * @param durationDTOList        持续时间dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param wellheadDTOList        井口dtolist
     * @param unOptimizeWellheadList 取消优化井口列表
     * @param durationMap            持续时间图
     * @param periodId               期间id
     * @param optimizeDate           优化日期
     */
    public void setUnOptimizeWellheadConfig(List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                            List<SpaceOptimizeWellheadDTO> wellheadDTOList, List<SpaceInstitutionWellheadView> unOptimizeWellheadList,
                                            Map<String, List<SpaceInstitutionDurationEnt>> durationMap, String periodId, DateTime optimizeDate) {
        if (CollUtil.isNotEmpty(unOptimizeWellheadList)) {
            for (SpaceInstitutionWellheadView wellhead : unOptimizeWellheadList) {
                String wellheadId = wellhead.getWellheadId();
                String wellNumber = wellhead.getWellNumber();
                if (CharSequenceUtil.isBlank(wellNumber) || CharSequenceUtil.isBlank(wellheadId)) {
                    continue;
                }
                String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellNumber, wellhead.getStartSeq(), optimizeDate, BusinessConstant.ONE);
                switch (wellhead.getIntervalTypeKey()) {
                    //小间开(不优化)
                    case "0":
                        if (CollUtil.isEmpty(durationMap) || !durationMap.containsKey(wellheadId)) {
                            continue;
                        }
                        //初始化未优化间开
                        List<SpaceInstitutionDurationEnt> durationList = durationMap.get(wellheadId);
                        for (SpaceInstitutionDurationEnt durationEnt : durationList) {
                            this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, optimizeDate);
                            this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                    durationEnt.getOpenWellTime(), durationEnt.getCloseWellTime(),
                                    BusinessConstant.ONE, optimizeDate
                            );
                        }
                        break;
                    //大间开
                    case "1":
                        //创建间开记录(全开)
                        this.createDuration(unOptimizeDurationList, recordId, periodId, wellheadId, BusinessConstant.ONE, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.END_OF_DAY_TIME, null,
                                BusinessConstant.ONE, optimizeDate);
                        this.createDuration(durationDTOList, recordId, periodId, wellheadId, BusinessConstant.ZERO, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.END_OF_DAY_TIME, null,
                                BusinessConstant.ONE, optimizeDate);
                        break;
                    default:
                        if (CharSequenceUtil.equals(wellhead.getRunTypeKey(), BusinessConstant.CONTINUOUS_PUMPING_WELL)) {
                            this.createDuration(unOptimizeDurationList, recordId, periodId, wellheadId, BusinessConstant.ONE, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.END_OF_DAY_TIME, null,
                                    BusinessConstant.ONE, optimizeDate);
                            this.createDuration(durationDTOList, recordId, periodId, wellheadId, BusinessConstant.ZERO, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.END_OF_DAY_TIME, null,
                                    BusinessConstant.ONE, optimizeDate);
                        } else {
                            //未配置全关
                            this.createDuration(unOptimizeDurationList, recordId, periodId, wellheadId, BusinessConstant.ONE, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.START_OF_DAY_TIME, null,
                                    BusinessConstant.ONE, optimizeDate);
                            this.createDuration(durationDTOList, recordId, periodId, wellheadId, BusinessConstant.ZERO, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.START_OF_DAY_TIME, null,
                                    BusinessConstant.ONE, optimizeDate);
                        }
                }
            }
        }
    }

    /**
     * 获得最大功率
     *
     * @param avgPowerList 平均功率列表
     * @return {@link BigDecimal}
     */
    private BigDecimal getPowerMax(List<DynamicQueryPlantPredictedPowerOutput> avgPowerList) {
        return avgPowerList.stream()
                .map(DynamicQueryPlantPredictedPowerOutput::getPower)
                .max(BigDecimal::compareTo)
                .orElse(BigDecimal.ZERO);
    }

    /**
     * 获取井口总功率
     *
     * @param spaceWellheadList 空间井口清单
     * @param detailId          详细信息id
     * @return {@link BigDecimal}
     */
    private BigDecimal getWellheadTotalPower(List<SpaceInstitutionWellheadView> spaceWellheadList, String detailId) {
        return spaceWellheadList.stream()
                .filter(w -> CharSequenceUtil.equals(detailId, w.getInstitutionId()))
                .map(SpaceInstitutionWellheadView::getServiceRating)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    /**
     * 获取当前制度关联的井口按照是否可优化分组
     *
     * @param spaceWellheadList 空间井口清单
     * @param detailId          详细信息id
     * @return {@link Map}<{@link Boolean}, {@link List}<{@link SpaceInstitutionWellheadView}>>
     */
    private Map<Boolean, List<SpaceInstitutionWellheadView>> getWellheadViewList(List<SpaceInstitutionWellheadView> spaceWellheadList, String detailId) {
        return spaceWellheadList.stream()
                .sorted(Comparator.comparing(SpaceInstitutionWellheadView::getStartSeq))
                .collect(
                        Collectors.partitioningBy(w -> CharSequenceUtil.equals(detailId, w.getInstitutionId()) &&
                                CharSequenceUtil.equals(BusinessConstant.INTERVAL_PUMPING_WELL, w.getRunTypeKey()) &&
                                CharSequenceUtil.equals("0", w.getIntervalTypeKey())
                        )
                );
    }

    /**
     * 获取井口间开时间段
     *
     * @param context  上下文
     * @param detailId 详细信息id
     * @return {@link Map}<{@link String}, {@link List}<{@link SpaceInstitutionDurationEnt}>>
     */
    private Map<String, List<SpaceInstitutionDurationEnt>> getDurationMap(XContext context, String detailId) {
        SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
        //通过间开ID和井口ID查所有井口时段配置
        List<SpaceInstitutionDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDurationEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceInstitutionDurationEnt::getInstitutionId, detailId)
                .orderByAsc(SpaceInstitutionDurationEnt::getOpenWellTime)
        );
        if (CollUtil.isEmpty(durationList)) {
            //没有设置时间段,无法优化
            return Collections.emptyMap();
        } else {
            for (SpaceInstitutionDurationEnt ent : durationList) {
                ent.setOpenTime(BaseUtils.strToDateTime(ent.getOpenWellTime()));
                ent.setCloseTime(BaseUtils.strToDateTime(ent.getCloseWellTime()));
            }
        }
        return durationList.stream()
                .collect(
                        Collectors.groupingBy(SpaceInstitutionDurationEnt::getWellheadId)
                );
    }

    /**
     * 按线路ID集合获取电站列表
     *
     * @param context 上下文
     * @param lineIds 线路ID
     * @return {@link List}<{@link DynamicQueryBasePowerLinePlantOutput}>
     */
    private List<DynamicQueryBasePowerLinePlantOutput> getPowerLinePlantListByLineIds(XContext context, Set<String> lineIds) {
        IBasePowerLineCloudService cloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLinePlantOutput> result = cloudService.queryPowerLinePlantListByParam(context,
                DynamicQueryBasePowerLinePlantInput.builder()
                        .lineIds(lineIds)
                        .build()
        );
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 条件查询线路关联储能设备参数平均值
     *
     * @param context 上下文
     * @param lineIds 线路ID
     * @return {@link Map}<{@link String}, {@link List}<{@link DynamicQueryBasePowerLineStorageViewOutput}>>
     */
    private Map<String, List<DynamicQueryBasePowerLineStorageViewOutput>> getPowerLineStorageAvgByLineIds(XContext context, Set<String> lineIds) {
        IBasePowerLineCloudService cloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineStorageViewOutput> result = cloudService.queryPowerLineStorageAvgByLineIds(context,
                DynamicQueryBasePowerLineStorageInput.builder()
                        .lineIds(lineIds)
                        .build()
        );
        result.throwIfFail();
        List<DynamicQueryBasePowerLineStorageViewOutput> list = result.getResult();
        Map<String, List<DynamicQueryBasePowerLineStorageViewOutput>> collect;
        if (CollUtil.isEmpty(list)) {
            collect = Collections.emptyMap();
        } else {
            collect = list.stream()
                    .collect(Collectors.groupingBy(DynamicQueryBasePowerLineStorageViewOutput::getLineId));
        }
        return collect;
    }

    /**
     * 查询线路下所有储能设备ID
     *
     * @param context 上下文
     * @param lineIds 线路ID
     * @return {@link Map}<{@link String}, {@link List}<{@link String}>>
     */
    private Map<String, List<String>> getPowerLineStorageListByLineIds(XContext context, Set<String> lineIds) {
        IBasePowerLineCloudService cloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineStorageOutput> result = cloudService.queryPowerLineStorageListByLineIds(context,
                DynamicQueryBasePowerLineStorageInput.builder()
                        .lineIds(lineIds)
                        .build());
        result.throwIfFail();
        List<DynamicQueryBasePowerLineStorageOutput> list = result.getResult();
        Map<String, List<String>> collect;
        if (CollUtil.isEmpty(list)) {
            collect = Collections.emptyMap();
        } else {
            collect = list.stream()
                    .collect(
                            Collectors.groupingBy(DynamicQueryBasePowerLineStorageOutput::getLineId,
                                    Collectors.mapping(DynamicQueryBasePowerLineStorageOutput::getStorageId, Collectors.toList()))
                    );
        }
        return collect;
    }

    /**
     * 为间开井口列表设置发电功率
     *
     * @param context           上下文
     * @param spaceWellheadList 空间井口清单
     */
    private void setServiceRatingForSpaceWellheadList(XContext context, List<SpaceInstitutionWellheadView> spaceWellheadList) {
        List<DynamicQueryBaseWellheadOutput> wellheadOutputList = ServiceUtil.getBaseWellList(context, DynamicQueryBaseWellheadInput.builder()
                .wellheadIds(spaceWellheadList.stream()
                        .map(SpaceInstitutionWellheadView::getWellheadId)
                        .collect(Collectors.toList())
                )
                .build());
        if (CollUtil.isEmpty(wellheadOutputList)) {
            return;
        }
        spaceWellheadList.forEach(s -> {
            s.setServiceRating(
                    wellheadOutputList.stream()
                            .filter(w -> CharSequenceUtil.equals(s.getWellheadId(), w.getId()))
                            .findAny()
                            .map(DynamicQueryBaseWellheadOutput::getServiceRating)
                            .orElse(BigDecimal.ZERO)
            );
        });
    }

    /**
     * 获取井口Map通过最大功率区分
     *
     * @param wellheadViewList 井口视图列表
     * @param powerMax         最大功率
     * @return {@link Map}<{@link Boolean}, {@link List}<{@link SpaceInstitutionWellheadView}>>
     */
    private Map<Boolean, List<SpaceInstitutionWellheadView>> getWellheadViewMapByPower(List<SpaceInstitutionWellheadView> wellheadViewList, BigDecimal powerMax) {
        return wellheadViewList.stream()
                .collect(
                        Collectors.partitioningBy(w -> powerMax.compareTo(w.getServiceRating()) >= BusinessConstant.ZERO)
                );
    }

    /**
     * 离网型间开记录
     *
     * @param offGridPeriodList 离网时段列表
     * @param wellheadId        井口id
     * @param openTime          断开时间
     * @param closeTime         关闭时间
     * @param firstStart        是否第一次启动
     */
    private void createDurationOffGridPeriod(List<SpaceOptimizeDurationDTO> offGridPeriodList, String wellheadId,
                                             Date openTime, Date closeTime, Integer firstStart) {
        SpaceOptimizeDurationDTO durationDTO = new SpaceOptimizeDurationDTO();
        durationDTO.setWellheadId(wellheadId);
        durationDTO.setOpenTime(openTime);
        durationDTO.setCloseTime(closeTime);
        durationDTO.setFirstStart(firstStart);
        offGridPeriodList.add(durationDTO);
    }

    /**
     * 是否在优化
     *
     * @param context  上下文
     * @param detailId 详细信息id
     * @param flag     旗帜
     * @param now      现在
     * @return boolean
     */
    private boolean isOptimize(XContext context, String detailId, Integer flag, DateTime now) {
        switch (flag) {
            case 0:
                SpaceOptimizeLongPeriodMapper periodMapper = context.getBean(SpaceOptimizeLongPeriodMapper.class);
                SpaceOptimizeLongPeriodEnt longPeriodEnt = periodMapper.selectOne(new QueryWrapper<SpaceOptimizeLongPeriodEnt>()
                        .select("INSTITUTION_ID",
                                "MAX( OPTIMIZE_DEADLINE ) AS OPTIMIZE_DEADLINE")
                        .lambda()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeLongPeriodEnt::getInstitutionId, detailId)
                        .groupBy(SpaceOptimizeLongPeriodEnt::getInstitutionId)
                );
                if (ObjectUtil.isNotNull(longPeriodEnt)) {
                    Date optimizeDead = longPeriodEnt.getOptimizeDeadline();
                    if (DateUtil.compare(now, optimizeDead) == -1) {
                        return true;
                    }
                }
                return false;
            case 1:
                SpaceOptimizeMidPeriodMapper midPeriodMapper = context.getBean(SpaceOptimizeMidPeriodMapper.class);
                SpaceOptimizeMidPeriodEnt midPeriodEnt = midPeriodMapper.selectOne(new QueryWrapper<SpaceOptimizeMidPeriodEnt>()
                        .select("INSTITUTION_ID",
                                "MAX( OPTIMIZE_DEADLINE ) AS OPTIMIZE_DEADLINE")
                        .lambda()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeMidPeriodEnt::getInstitutionId, detailId)
                        .groupBy(SpaceOptimizeMidPeriodEnt::getInstitutionId)
                );
                if (ObjectUtil.isNotNull(midPeriodEnt)) {
                    Date optimizeDead = midPeriodEnt.getOptimizeDeadline();
                    if (DateUtil.compare(now, optimizeDead) == -1) {
                        return true;
                    }
                }
                return false;
            case 2:
                SpaceOptimizeUltraPeriodMapper ultraPeriodMapper = context.getBean(SpaceOptimizeUltraPeriodMapper.class);
                SpaceOptimizeUltraPeriodEnt ultraPeriodEnt = ultraPeriodMapper.selectOne(new QueryWrapper<SpaceOptimizeUltraPeriodEnt>()
                        .select("INSTITUTION_ID",
                                "MAX( OPTIMIZE_DEADLINE ) AS OPTIMIZE_DEADLINE")
                        .lambda()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeUltraPeriodEnt::getInstitutionId, detailId)
                        .groupBy(SpaceOptimizeUltraPeriodEnt::getInstitutionId)
                );
                if (ObjectUtil.isNotNull(ultraPeriodEnt)) {
                    Date optimizeDead = ultraPeriodEnt.getOptimizeDeadline();
                    if (DateUtil.compare(now, optimizeDead) == -1) {
                        return true;
                    }
                }
            default:
                return false;
        }
    }

    /**
     * 加时
     *
     * @param durationDTO 持续时间dto
     * @param endTime     结束时间
     * @param minute      分钟
     */
    private void overtime(SpaceOptimizeDurationDTO durationDTO, DateTime endTime, int minute) {
        //判断向前补还是向后补
        if (DateUtil.compare(durationDTO.getOpenTime(), endTime) == 0) {
            //向前补
            durationDTO.setOpenTime(DateUtil.offsetMinute(durationDTO.getOpenTime(), -minute));
        } else {
            //向后补
            durationDTO.setCloseTime(DateUtil.offsetMinute(durationDTO.getCloseTime(), minute));
        }
    }

    /**
     * 通过时段类型获取权重
     *
     * @param periodTypeKey 周期类型键
     * @return int
     */
    public int getWeightByPeriodTypeKey(String periodTypeKey) {
        switch (periodTypeKey) {
            case BusinessConstant.PERIOD_TYPE_LOW:
                return 5;
            case "PEACE":
                return 3;
            case "PEAK":
                return 1;
            case "RUSH":
            default:
                return 0;
        }
    }

    /**
     * 划分区域列表
     *
     * @param serviceRating 服务等级
     * @param rangeToList   要列出范围
     * @param powerList     电源列表
     * @param strategyList  策略列表
     * @return {@link List}<{@link SpaceOptimizeWeight}>
     */
    private List<SpaceOptimizeWeight> zoningList(BigDecimal serviceRating, List<DateTime> rangeToList, List<DynamicQueryPlantPredictedPowerOutput> powerList, List<GetBasePriceStrategyDetailOutput> strategyList) {
        int rangeListSize = rangeToList.size();
        int rangeIndex = 0;
        int strategyIndex = 0;
        Map<Date, DynamicQueryPlantPredictedPowerOutput> powerMap;
        if (CollUtil.isNotEmpty(powerList)) {
            powerMap = powerList.stream()
                    .collect(Collectors.toMap(DynamicQueryPlantPredictedPowerOutput::getCreateTime, Function.identity()));
        } else {
            powerMap = Collections.emptyMap();
        }
        List<SpaceOptimizeWeight> weightList = new ArrayList<>(rangeListSize);
        DynamicQueryPlantPredictedPowerOutput firstPower;
        GetBasePriceStrategyDetailOutput firstStrategy;
        while (rangeListSize > 0 && rangeListSize > rangeIndex) {
            boolean powerFlag = false;
            SpaceOptimizeWeight firstWeight = new SpaceOptimizeWeight();
            DateTime firstTime = rangeToList.get(rangeIndex);
            firstWeight.setTimestamp(firstTime);
            firstWeight.setTimeString(firstTime.toString());
            firstWeight.setSort(rangeIndex);
            if (powerMap.containsKey(firstTime)) {
                firstPower = powerMap.get(firstTime);
                firstWeight.setPower(firstPower.getPower());
                //判断发电量是否满足运行功率
                powerFlag = firstPower.getPower().compareTo(BigDecimal.ZERO) > 0;
                if (powerFlag) {
                    firstWeight.setWeight(10);
                    firstWeight.setGenerationTypeKey(BusinessConstant.PHOTOVOLTAIC);
                }
            }
            firstStrategy = strategyList.get(strategyIndex);
            if (ObjectUtil.isNull(firstStrategy.getCloseTime()) && CharSequenceUtil.equals(firstStrategy.getEndTime(), BusinessConstant.END_OF_DAY_TIME)) {
                firstStrategy.setCloseTime(BusinessConstant.DATE_FLAG);
            }
            //在市电峰谷时段内,且未满足运行功率,设置权重
            if (!powerFlag && DateUtil.isIn(firstTime, firstStrategy.getOpenTime(), ObjectUtil.isEmpty(firstStrategy.getCloseTime()) && CharSequenceUtil.equals(BusinessConstant.END_OF_DAY_TIME, firstStrategy.getStartTime()) ? BusinessConstant.DATE_FLAG : firstStrategy.getCloseTime())) {
                if (CharSequenceUtil.equals(firstStrategy.getPeriodTypeKey(), "RUSH")) {
                    //尖峰时段要停井
                    firstWeight.setWeight(0);
                } else if (!powerFlag) {
                    firstWeight.setWeight(this.getWeightByPeriodTypeKey(firstStrategy.getPeriodTypeKey()));
                }
            }
            if (firstTime.compareTo(firstStrategy.getCloseTime()) >= 0) {
                strategyIndex++;
            }
            weightList.add(firstWeight);
            rangeIndex++;
        }
        //排序
        weightList.sort(Comparator.comparing(SpaceOptimizeWeight::getSort));
        return weightList;
    }

    /**
     * 计算权重列表
     *
     * @param serviceRating 服务等级
     * @param rangeToList   要列出范围
     * @param powerList     电源列表
     * @param strategyList  策略列表
     * @param isOffGrid     是否离网型
     * @return {@link List}<{@link SpaceOptimizeWeight}>
     */
    private List<SpaceOptimizeWeight> calculateWeightList(BigDecimal serviceRating, List<DateTime> rangeToList, List<DynamicQueryPlantPredictedPowerOutput> powerList, List<GetBasePriceStrategyDetailOutput> strategyList, boolean isOffGrid) {
        int rangeListSize = rangeToList.size();
        int powerListSize = powerList.size();
        int strategyListSize = isOffGrid ? 0 : strategyList.size();
        int rangeIndex = 0;
        int powerIndex = 0;
        int strategyIndex = 0;
        List<SpaceOptimizeWeight> weightList = new ArrayList<>(rangeListSize);
        DynamicQueryPlantPredictedPowerOutput firstPower;
        DynamicQueryPlantPredictedPowerOutput lastPower;
        GetBasePriceStrategyDetailOutput firstStrategy;
        GetBasePriceStrategyDetailOutput lastStrategy;
        while (rangeListSize > 0 && rangeListSize > rangeIndex && powerListSize > powerIndex) {
            boolean powerFlag = false;
            SpaceOptimizeWeight firstWeight = new SpaceOptimizeWeight();
            DateTime firstTime = rangeToList.get(rangeIndex);
            firstWeight.setTimestamp(firstTime);
            firstWeight.setSort(rangeIndex);
            firstPower = powerList.get(powerIndex);
            if (0 == firstTime.compareTo(firstPower.getCreateTime())) {
                firstWeight.setPower(firstPower.getPower());
                //判断发电量是否满足运行功率
                powerFlag = firstPower.getPower().compareTo(serviceRating) >= 0;
                if (powerFlag) {
                    firstWeight.setWeight(10);
                    firstWeight.setGenerationTypeKey(BusinessConstant.PHOTOVOLTAIC);
                }
                powerIndex++;
            }
            if (isOffGrid) {
                if (!powerFlag) {
                    firstWeight.setWeight(1);
                    firstWeight.setGenerationTypeKey(BusinessConstant.STORED_ENERGY);
                }
            } else {
                firstStrategy = strategyList.get(strategyIndex);
                Date closeTime = ObjectUtil.isEmpty(firstStrategy.getCloseTime()) && CharSequenceUtil.equals(BusinessConstant.END_OF_DAY_TIME, firstStrategy.getStartTime()) ? BusinessConstant.DATE_FLAG : firstStrategy.getCloseTime();
                //在市电峰谷时段内,且未满足运行功率,设置权重
                if (DateUtil.isIn(firstTime, firstStrategy.getOpenTime(), closeTime)) {
                    if (CharSequenceUtil.equals(firstStrategy.getPeriodTypeKey(), "RUSH")) {
                        //尖峰时段要停井
                        firstWeight.setWeight(0);
                    } else if (!powerFlag) {
                        firstWeight.setWeight(this.getWeightByPeriodTypeKey(firstStrategy.getPeriodTypeKey()));
                    }
                }
                if (firstTime.compareTo(firstStrategy.getCloseTime()) >= 0) {
                    strategyIndex++;
                }
            }
            weightList.add(firstWeight);
            rangeIndex++;
            rangeListSize--;
            //防止重复插入
            if (rangeIndex >= rangeListSize) {
                continue;
            }
            /*----------------------------------------------------*/
            powerFlag = false;
            SpaceOptimizeWeight lastWeight = new SpaceOptimizeWeight();
            DateTime lastTime = rangeToList.get(rangeListSize);
            lastWeight.setTimestamp(lastTime);
            lastWeight.setSort(rangeListSize);
            lastPower = powerList.get(powerListSize - 1);
            if (0 == lastTime.compareTo(lastPower.getCreateTime())) {
                lastWeight.setPower(lastPower.getPower());
                //判断发电量是否满足运行功率
                powerFlag = lastPower.getPower().compareTo(serviceRating) >= 0;
                if (powerFlag) {
                    lastWeight.setWeight(10);
                    lastWeight.setGenerationTypeKey(BusinessConstant.PHOTOVOLTAIC);
                }
                powerListSize--;
            }
            if (isOffGrid) {
                if (!powerFlag) {
                    lastWeight.setWeight(1);
                    lastWeight.setGenerationTypeKey(BusinessConstant.STORED_ENERGY);
                }
            } else {
                lastStrategy = strategyList.get(strategyListSize - 1);
                Date closeTime = ObjectUtil.isEmpty(lastStrategy.getCloseTime()) && CharSequenceUtil.equals(BusinessConstant.END_OF_DAY_TIME, lastStrategy.getEndTime()) ? BusinessConstant.DATE_FLAG : lastStrategy.getCloseTime();
                //在市电峰谷时段内,且未满足运行功率,设置权重
                if (DateUtil.isIn(lastTime, lastStrategy.getOpenTime(), closeTime)) {
                    if (CharSequenceUtil.equals(lastStrategy.getPeriodTypeKey(), "RUSH")) {
                        //尖峰时段要停井
                        lastWeight.setWeight(0);
                    } else if (!powerFlag) {
                        lastWeight.setWeight(this.getWeightByPeriodTypeKey(lastStrategy.getPeriodTypeKey()));
                    }
                }
                if (lastTime.compareTo(lastStrategy.getOpenTime()) <= 0) {
                    strategyListSize--;
                }
            }
            weightList.add(lastWeight);
        }
        //排序
        weightList.sort(Comparator.comparing(SpaceOptimizeWeight::getSort));
        return weightList;
    }

    /**
     * 权重时间处理
     *
     * @param weightList    重量清单
     * @param minOpenMinute 最小开放分钟
     * @return {@link List}<{@link SpaceOptimizeWeightDuration}>
     */
    private List<SpaceOptimizeWeightDuration> weightListProcessing(List<SpaceOptimizeWeight> weightList, int minOpenMinute) {
        SpaceOptimizeWeight weight;
        //分级取时间段
        List<SpaceOptimizeWeightDuration> weightDurationList = new ArrayList<>(32);
        long between;
        for (int i1 = 0; i1 < BusinessConstant.LADDER.length; i1++) {
            int begin = -1;
            //取每级时间
            for (int i2 = 0; i2 < weightList.size(); i2++) {
                weight = weightList.get(i2);
                //过滤条件:权重相同
                if (BusinessConstant.LADDER[i1] == weight.getWeight()) {
                    //确定开始时间位置
                    if (begin == -1) {
                        begin = i2;
                    }
                } else if (begin != -1) {
                    //如果开井时间不满足最小开井时间则舍弃
                    between = DateUtil.between(weightList.get(begin).getTimestamp(), weightList.get(i2).getTimestamp(), DateUnit.MINUTE);
                    //创建区间
                    weightDurationList.add(
                            SpaceOptimizeWeightDuration.builder()
                                    .openTime(weightList.get(begin).getTimestamp())
                                    .closeTime(weightList.get(i2).getTimestamp())
                                    .openIndex(begin)
                                    .closeIndex(i2)
                                    .duration(between)
                                    .weight(BusinessConstant.LADDER[i1])
                                    .build()
                    );
                    begin = -1;
                }
            }
        }
        //时间处理并排序(处理后第一次开井时间的索引位置为0)
        weightDurationList.sort((o1, o2) -> o2.getWeight() - o1.getWeight());
        return weightDurationList;
    }

    /**
     * 时间段优化
     *
     * @param optimizeDurationDTOList 优化工期dtolist
     * @param weightDurationList      重量持续时间列表
     * @param firstOpenWellTime       首次开井时间
     * @param i                       我
     * @param startInterval           开始间隔
     * @param minOpenMinute           最小开放分钟
     * @param wellhead                井口
     * @param weightList              重量清单
     * @return {@link DateTime}
     */
    private DateTime timePeriodOptimization(List<SpaceOptimizeDurationDTO> optimizeDurationDTOList, List<SpaceOptimizeWeightDuration> weightDurationList, DateTime firstOpenWellTime,
                                            int i, Integer startInterval, int minOpenMinute, SpaceInstitutionWellheadView wellhead, List<SpaceOptimizeWeight> weightList) {
        //每日开井时长
        int dayOpenMinute = wellhead.getRunDuration()
                .multiply(BusinessConstant.BIG_DECIMAL_60)
                .intValue();
        //每日最大开井时长(分钟)
        int maxOpenMinute = wellhead.getMaxOpen()
                .multiply(BusinessConstant.BIG_DECIMAL_60)
                .intValue();
        //每日最小关井时长(分钟)
        int minCloseMinute = wellhead.getMinClose()
                .multiply(BusinessConstant.BIG_DECIMAL_60)
                .intValue();
        //每日最大关井时长(分钟)
        int maxCloseMinute = wellhead.getMaxClose()
                .multiply(BusinessConstant.BIG_DECIMAL_60)
                .intValue();
        //总开井时间
        long sumOpenTime = 0;
        for (int i1 = 0; i1 < weightDurationList.size(); i1++) {
            SpaceOptimizeWeightDuration weightDuration = weightDurationList.get(i1);
            long duration = weightDuration.getDuration();
            DateTime openTime = weightDuration.getOpenTime();
            //偏移其他井口开井时间,并修改时间间隔
            if (i > 0 && 0 == i1 && 0 >= DateUtil.compare(openTime, firstOpenWellTime)) {
                duration -= (long) startInterval * i;
                openTime = DateUtil.offsetMinute(openTime, startInterval * i);
            }
            DateTime closeTime = weightDuration.getCloseTime();
            //正向标识
            boolean forwardFlag = true;
            //偏移开/关井时间
            if (CollUtil.isNotEmpty(optimizeDurationDTOList)) {
                SpaceOptimizeDurationDTO durationDTO = optimizeDurationDTOList.get(optimizeDurationDTOList.size() - 1);
                Date firstOpenTime = optimizeDurationDTOList.get(0).getOpenTime();
                Date lastCloseTime = durationDTO.getCloseTime();
                long l;
                //如果关井时间在第一次开井时间之前,判断是否需要偏移关井时间
                if (DateUtil.compare(firstOpenTime, closeTime) >= 0) {
                    forwardFlag = false;
                    l = closeTime.between(firstOpenTime, DateUnit.MINUTE);
                    long l1 = l - minCloseMinute;
                    //小于0,说明不满足最小停井时间,需要补
                    if (l1 < 0) {
                        //偏移关井时间
                        closeTime = closeTime.offsetNew(DateField.MINUTE, (int) l1);
                        duration += l1;
                        if (duration < minOpenMinute) {
                            continue;
                        }
                    } else if (l1 > 0) {
                        //判断时间间隔是否超过最大停井时间
                        if (l > maxCloseMinute) {
                            //调整当前权重
                            if (i1 > 0) {
                                //计算出本次应该的开/关井时间
                                closeTime = DateUtil.offsetMinute(closeTime, (int) l - maxCloseMinute);
                                openTime = DateUtil.offsetMinute(openTime, (int) l - maxCloseMinute);
                            }
                        }
                    }
                } else {
                    //计算当前开井时间和上一次关井时间的时间间隔
                    l = openTime.between(lastCloseTime, DateUnit.MINUTE);
                    //比较时间
                    if (DateUtil.compare(lastCloseTime, openTime) > 0) {
                        //时间间隔-(超出部分时间+最小关井时长)
                        duration -= (l + minCloseMinute);
                        if (duration >= minOpenMinute) {
                            //如果剩余时长能满足最小开井时长,则计算开井时间(用关井时间往前推)
                            openTime = closeTime.offsetNew(DateField.MINUTE, (int) -duration);
                        } else {
                            continue;
                        }
                    } else {
                        long l1 = l - minCloseMinute;
                        //小于0,说明不满足最小停井时间,需要补
                        if (l1 < 0) {
                            openTime = openTime.offsetNew(DateField.MINUTE, (int) -l1);
                            //偏移启动时间,判断时间区间是否满足最小开井时长
                            duration += l1;
                            if (duration < minOpenMinute) {
                                continue;
                            }
                        } else if (l1 > 0) {
                            //判断时间间隔是否超过最大停井时间
                            if (l > maxCloseMinute) {
                                //调整当前权重
                                if (i1 > 0) {
                                    //计算出本次应该的开井时间
                                    openTime = DateUtil.offsetMinute(lastCloseTime, maxCloseMinute);
                                    //修改本次区间的开始时间及时间间隔
                                    duration = DateUtil.between(openTime, weightDuration.getCloseTime(), DateUnit.MINUTE);
                                }
                            }
                        }
                    }
                }
            }
            //判断时间间隔是否能满足最大开井时间
            if (duration >= maxOpenMinute) {
                //满足,判断能满足几次(最大开井时间)
                //todo : 能力有限,只能用(最大开井时间+最小停井时间)固定范围,求不了最优排布
                int div = (int) NumberUtil.div(duration, maxOpenMinute, 0, RoundingMode.UP);
                if (forwardFlag) {
                    //正向拆分
                    for (int i2 = 1; i2 <= div; i2++) {
                        DateTime closeTimeNew;
                        if (duration >= maxOpenMinute) {
                            sumOpenTime += maxOpenMinute;
                            closeTimeNew = openTime.offsetNew(DateField.MINUTE, maxOpenMinute);
                            duration = duration - maxOpenMinute - minCloseMinute;
                        } else if (duration >= minOpenMinute) {
                            sumOpenTime += duration;
                            closeTimeNew = openTime.offsetNew(DateField.MINUTE, (int) duration);
                            duration = -minCloseMinute;
                        } else {
                            //时间不够,舍弃
                            continue;
                        }
                        optimizeDurationDTOList.add(SpaceOptimizeDurationDTO.builder()
                                .openTime(openTime)
                                .closeTime(closeTimeNew)
                                .build());
                        //下次启动时间为本次关井时间向后移动最小关井时间
                        openTime = closeTimeNew.offsetNew(DateField.MINUTE, minCloseMinute);
                    }
                } else {
                    //反向拆分
                    for (int i2 = 1; i2 <= div; i2++) {
                        DateTime openTimeNew;
                        if (duration >= maxOpenMinute) {
                            sumOpenTime += maxOpenMinute;
                            openTimeNew = closeTime.offsetNew(DateField.MINUTE, -maxOpenMinute);
                            duration = duration - maxOpenMinute - minCloseMinute;
                        } else if (duration >= minOpenMinute) {
                            sumOpenTime += duration;
                            openTimeNew = closeTime.offsetNew(DateField.MINUTE, -(int) duration);
                            duration = -minCloseMinute;
                        } else {
                            //时间不够,舍弃
                            continue;
                        }
                        optimizeDurationDTOList.add(SpaceOptimizeDurationDTO.builder()
                                .openTime(openTimeNew)
                                .closeTime(closeTime)
                                .build());
                        //下次启动时间为本次关井时间向后移动最小关井时间
                        closeTime = openTimeNew.offsetNew(DateField.MINUTE, -minCloseMinute);
                    }
                }
            } else {
                //不满足,取全部
                optimizeDurationDTOList.add(SpaceOptimizeDurationDTO.builder()
                        .openTime(openTime)
                        .closeTime(closeTime)
                        .build());
                sumOpenTime += duration;
            }
            //判断开井总时间是否大于每日最大开井时间
            if (sumOpenTime >= dayOpenMinute) {
                long outdo = sumOpenTime - dayOpenMinute;
                if (outdo > 0) {
                    SpaceOptimizeDurationDTO durationDTO = optimizeDurationDTOList.get(optimizeDurationDTOList.size() - 1);
                    if (forwardFlag) {
                        DateTime closeTimeNew = DateUtil.offsetMinute(durationDTO.getCloseTime(), (int) -outdo);
                        int compare = closeTimeNew.compareTo(durationDTO.getOpenTime());
                        if (compare <= 0) {
                            optimizeDurationDTOList.remove(optimizeDurationDTOList.size() - 1);
                            sumOpenTime -= outdo;
                        } else {
                            if (DateUtil.between(durationDTO.getOpenTime(), closeTimeNew, DateUnit.MINUTE) < minOpenMinute) {
                                //如果缩短后时间间隔不满足最小开井时长则删除
                                sumOpenTime -= DateUtil.between(durationDTO.getOpenTime(), durationDTO.getCloseTime(), DateUnit.MINUTE);
                                optimizeDurationDTOList.remove(optimizeDurationDTOList.size() - 1);
                            } else {
                                //需要补时间,下面统一补
                                durationDTO.setCloseTime(closeTimeNew);
                            }
                        }
                    } else {
                        DateTime openTimeNew = DateUtil.offsetMinute(durationDTO.getOpenTime(), (int) outdo);
                        int compare = openTimeNew.compareTo(durationDTO.getCloseTime());
                        if (compare >= 0) {
                            optimizeDurationDTOList.remove(optimizeDurationDTOList.size() - 1);
                            sumOpenTime -= outdo;
                        } else {
                            if (DateUtil.between(openTimeNew, durationDTO.getCloseTime(), DateUnit.MINUTE) < minOpenMinute) {
                                //如果缩短后时间间隔不满足最小开井时长则删除
                                sumOpenTime -= DateUtil.between(durationDTO.getOpenTime(), durationDTO.getCloseTime(), DateUnit.MINUTE);
                                optimizeDurationDTOList.remove(optimizeDurationDTOList.size() - 1);
                            } else {
                                //需要补时间,下面统一补
                                durationDTO.setOpenTime(openTimeNew);
                            }
                        }
                    }
                }
                break;
            }
            //重新按开井时间排序
            optimizeDurationDTOList.sort(Comparator.comparing(SpaceOptimizeDurationDTO::getOpenTime));
            //记录上一次第一次开井时间
            if (0 == i1) {
                firstOpenWellTime = DateUtil.date(optimizeDurationDTOList.get(0).getOpenTime());
            }
        }
        //判断是否需要补时间
        if (sumOpenTime < dayOpenMinute) {
            //创建需要补时间的时间范围
            List<SpaceOptimizeDurationDTO> replenishList = new ArrayList<>(12);
            int size = optimizeDurationDTOList.size();
            //记录下一个索引位置
            int indexNextRecord = 0;
            //记录上一个索引位置
            int indexLastRecord;
            for (int i1 = 0; i1 < size; i1++) {
                //计算需要补的时间
                long wantMinute = dayOpenMinute - sumOpenTime;
                //取当前结果
                SpaceOptimizeDurationDTO current = optimizeDurationDTOList.get(i1);
                Date currentOpenTime = current.getOpenTime();
                Date currentCloseTime = current.getCloseTime();
                //计算开井时长
                long openMinute = DateUtil.between(currentOpenTime, currentCloseTime, DateUnit.MINUTE);
                //取下一段
                if (indexNextRecord == size - 1) {
                    //如果到最后,下一段则取开头
                    indexNextRecord = 0;
                } else {
                    indexNextRecord++;
                }
                SpaceOptimizeDurationDTO next = optimizeDurationDTOList.get(indexNextRecord);
                Date nextOpenTime = next.getOpenTime();
                //计算和下一段的关井时长
                long nextCloseMinute;
                if (indexNextRecord == 0) {
                    //跨天计算
                    nextCloseMinute = DateUtil.between(currentCloseTime, DateUtil.offsetDay(nextOpenTime, 1), DateUnit.MINUTE);
                } else {
                    nextCloseMinute = DateUtil.between(currentCloseTime, nextOpenTime, DateUnit.MINUTE);
                }
                //取上一段
                indexLastRecord = i1 - 1;
                if (indexLastRecord == -1) {
                    //如果到最后,下一段则取开头
                    indexLastRecord = size - 1;
                }
                SpaceOptimizeDurationDTO last = optimizeDurationDTOList.get(indexLastRecord);
                Date lastCloseTime = last.getCloseTime();
                //计算和上一段的关井时长
                long lastCloseMinute;
                if (indexLastRecord == size - 1) {
                    //跨天计算
                    lastCloseMinute = DateUtil.between(currentOpenTime, DateUtil.offsetDay(lastCloseTime, -1), DateUnit.MINUTE);
                } else {
                    lastCloseMinute = DateUtil.between(currentOpenTime, lastCloseTime, DateUnit.MINUTE);
                }
                //----------------------------------数据处理----------------------------------
                //计算下一段可优化开井时长
                long nextWantMinute = this.getCanMinute(nextCloseMinute, minCloseMinute, wantMinute);
                //计算上一段可优化开井时长
                long lastWantMinute = this.getCanMinute(lastCloseMinute, minCloseMinute, wantMinute);
                //记录(左右移动标识位,可补时间)
                boolean rightFlag = false;
                boolean leftFlag = false;
                long rightMinute = 0;
                long leftMinute = 0;
                //当前开井时长能否满足最小开井时长
                if (openMinute >= minOpenMinute) {
                    //还能开多少
                    long spaceMinute = maxOpenMinute - openMinute;
                    if (spaceMinute <= 0) {
                        //没有优化空间 todo:研究研究是否需要往回缩 往回缩spaceMinute为负数
                        continue;
                    }
                    //补齐:先补小
                    boolean isNext = nextWantMinute < lastWantMinute;
                    if (isNext && nextWantMinute > 0) {
                        rightFlag = true;
                        if (nextWantMinute >= spaceMinute) {
                            //在范围内,直接补
                            rightMinute = spaceMinute;
                        } else {
                            rightMinute = nextWantMinute;
                        }
                    } else if (!isNext && lastWantMinute > 0) {
                        leftFlag = true;
                        if (lastWantMinute >= spaceMinute) {
                            //在范围内,直接补
                            leftMinute = spaceMinute;
                        } else {
                            leftMinute = lastWantMinute;
                        }
                    }
                    //补大:谁大取谁 ---- 计算一大一小,不冲突,不用重置变量
                    if (nextWantMinute > lastWantMinute) {
                        //往后移关井时间
                        rightFlag = true;
                        if (nextWantMinute >= spaceMinute) {
                            //在范围内,直接补
                            rightMinute = spaceMinute;
                        } else {
                            rightMinute = nextWantMinute;
                        }
                    } else {
                        //往前移开井时间
                        leftFlag = true;
                        if (lastWantMinute >= spaceMinute) {
                            //在范围内,直接补
                            leftMinute = spaceMinute;
                        } else {
                            leftMinute = lastWantMinute;
                        }
                    }
                } else {
                    //不满足最小开井时长,计算一个可补时间的区间
                    long minLackMinute = minOpenMinute - openMinute;
                    long maxLackMinute = maxOpenMinute - openMinute;
                    //补齐:先补小
                    boolean isNext = nextWantMinute < lastWantMinute;
                    if (isNext && nextWantMinute > 0) {
                        rightFlag = true;
                        if (nextWantMinute >= minLackMinute && nextWantMinute <= maxLackMinute) {
                            //在区间范围内,直接补
                            rightMinute = nextWantMinute;
                        } else {
                            if (nextWantMinute >= minLackMinute) {
                                rightMinute = minLackMinute;
                            } else {
                                rightMinute = nextWantMinute;
                            }
                        }
                    } else if (!isNext && lastWantMinute > 0) {
                        leftFlag = true;
                        if (lastWantMinute >= minLackMinute && lastWantMinute <= maxLackMinute) {
                            //在区间范围内,直接补
                            leftMinute = lastWantMinute;
                        } else {
                            if (lastWantMinute >= minLackMinute) {
                                leftMinute = minLackMinute;
                            } else {
                                leftMinute = lastWantMinute;
                            }
                        }
                    }
                    //补大:谁大取谁 ---- 计算一大一小,不冲突,不用重置变量
                    if (nextWantMinute > lastWantMinute) {
                        //往后移关井时间
                        rightFlag = true;
                        if (nextWantMinute >= minLackMinute && nextWantMinute <= maxLackMinute) {
                            //在区间范围内,直接补
                            rightMinute = nextWantMinute;
                        } else {
                            if (nextWantMinute >= maxLackMinute) {
                                rightMinute = maxLackMinute;
                            } else {
                                rightMinute = nextWantMinute;
                            }
                        }
                    } else {
                        //往前移开井时间
                        leftFlag = true;
                        if (lastWantMinute >= minLackMinute && lastWantMinute <= maxLackMinute) {
                            //在区间范围内,直接补
                            leftMinute = lastWantMinute;
                        } else {
                            if (lastWantMinute >= maxLackMinute) {
                                leftMinute = maxLackMinute;
                            } else {
                                leftMinute = lastWantMinute;
                            }
                        }
                    }
                }
                //算时间补,修改总时间
                DateTime optimizeTime;
                if (rightFlag) {
                    optimizeTime = DateUtil.offsetMinute(currentCloseTime, (int) rightMinute);
                    if (DateUtil.compare(optimizeTime, BusinessConstant.DATE_FLAG) >= 0) {
                        //跨天转换,加到优化数组里
                        SpaceOptimizeDurationDTO copy = XCopyUtils.copyNewObject(current, SpaceOptimizeDurationDTO.class);
                        copy.setOpenTime(BusinessConstant.DATE_FLAG_BEGIN);
                        copy.setCloseTime(DateUtil.offsetDay(optimizeTime, -1));
                        replenishList.add(copy);
                        //重置结束时间到第二天0点
                        optimizeDurationDTOList.get(i1).setCloseTime(BusinessConstant.DATE_FLAG);
                    } else {
                        optimizeDurationDTOList.get(i1).setCloseTime(optimizeTime);
                    }
                    dayOpenMinute = dayOpenMinute - (int) rightMinute;
                }
                if (leftFlag) {
                    optimizeTime = DateUtil.offsetMinute(currentOpenTime, (int) -leftMinute);
                    if (DateUtil.compare(optimizeTime, BusinessConstant.DATE_FLAG_BEGIN) <= 0) {
                        //跨天转换,加到优化数组里
                        SpaceOptimizeDurationDTO copy = XCopyUtils.copyNewObject(current, SpaceOptimizeDurationDTO.class);
                        copy.setOpenTime(DateUtil.offsetDay(optimizeTime, 1));
                        copy.setCloseTime(BusinessConstant.DATE_FLAG);
                        replenishList.add(copy);
                        //重置开始时间到第一天0点
                        optimizeDurationDTOList.get(i1).setOpenTime(BusinessConstant.DATE_FLAG_BEGIN);
                    } else {
                        optimizeDurationDTOList.get(i1).setOpenTime(optimizeTime);
                    }
                    dayOpenMinute = dayOpenMinute - (int) leftMinute;
                }
            }
            //将优化结果加入结果集中,并重新排序
            if (CollUtil.isNotEmpty(replenishList)) {
                //记录未使用的间开
                List<SpaceOptimizeDurationDTO> tempList = new ArrayList<>(12);
                //标识:0_未使用,1_开井用了,2_关井用了
                int isIn;
                for (SpaceOptimizeDurationDTO rep : replenishList) {
                    Date repOpenTime = rep.getOpenTime();
                    Date repCloseTime = rep.getCloseTime();
                    isIn = 0;
                    for (SpaceOptimizeDurationDTO dto : optimizeDurationDTOList) {
                        Date openTime = dto.getOpenTime();
                        Date closeTime = dto.getCloseTime();
                        if (DateUtil.isIn(repOpenTime, openTime, closeTime) && DateUtil.compare(repCloseTime, closeTime) >= 0) {
                            dto.setCloseTime(repCloseTime);
                            if (isIn == 1) {
                                continue;
                            }
                            isIn = 1;
                        }
                        if (DateUtil.isIn(repCloseTime, openTime, closeTime) && DateUtil.compare(repOpenTime, openTime) <= 0) {
                            dto.setOpenTime(repOpenTime);
                            if (isIn == 2) {
                                continue;
                            }
                            isIn = 2;
                        }
                    }
                    if (isIn == 0) {
                        tempList.add(rep);
                    }
                }
                if (CollUtil.isNotEmpty(tempList)) {
                    for (SpaceOptimizeDurationDTO temp : tempList) {
                        optimizeDurationDTOList.add(temp);
                    }
                }
            }
        }
        return firstOpenWellTime;
    }

    /**
     * 可优化时长计算
     *
     * @param closeMinute    关闭分钟
     * @param minCloseMinute 分钟关闭分钟
     * @param wantMinute     需要补的总时间
     * @return long
     */
    private long getCanMinute(long closeMinute, long minCloseMinute, long wantMinute) {
        long canMinute;
        if (closeMinute == minCloseMinute) {
            //满足最小关井时长舍弃
            canMinute = 0;
        } else if (closeMinute > minCloseMinute) {
            //关井时长比最小设定值大
            canMinute = closeMinute - minCloseMinute;
        } else {
            //不满足最小关井时长,能否合并
            canMinute = closeMinute;
        }
        if (canMinute > wantMinute) {
            canMinute = wantMinute;
        }
        if (wantMinute <= 0) {
            canMinute = 0;
        }
        return canMinute;
    }

    /**
     * 添加优化结果
     *
     * @param optimizeDurationDTOList 优化工期dtolist
     * @param durationDTOList         持续时间dtolist
     * @param tundraStrategyList      苔原战略清单
     * @param periodId                期间id
     * @param recordId                记录id
     * @param wellheadId              井口id
     * @param startDate               开始日期
     */
    private void addOptimizeDuration(List<SpaceOptimizeDurationDTO> optimizeDurationDTOList, List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> tundraStrategyList,
                                     String periodId, String recordId, String wellheadId, DateTime startDate) {
        if (CollUtil.isNotEmpty(optimizeDurationDTOList)) {
            //重新按开井时间排序
            optimizeDurationDTOList.sort(Comparator.comparing(SpaceOptimizeDurationDTO::getOpenTime));
            //时间段优化
            List<SpaceOptimizeDurationDTO> newDurationDTOList = new ArrayList<>(optimizeDurationDTOList.size());
            newDurationDTOList.add(optimizeDurationDTOList.get(0));
            for (int i = 1; i < optimizeDurationDTOList.size(); i++) {
                SpaceOptimizeDurationDTO nowOne = optimizeDurationDTOList.get(i);
                SpaceOptimizeDurationDTO lastOne = newDurationDTOList.get(newDurationDTOList.size() - 1);
                if (0 == DateUtil.compare(lastOne.getCloseTime(), nowOne.getOpenTime())) {
                    lastOne.setCloseTime(nowOne.getCloseTime());
                } else {
                    newDurationDTOList.add(nowOne);
                }
            }
            optimizeDurationDTOList = newDurationDTOList;
            SpaceOptimizeDurationDTO durationDTO;
            for (int i1 = 0; i1 < optimizeDurationDTOList.size(); i1++) {
                durationDTO = optimizeDurationDTOList.get(i1);
                DateTime startOffset = DateUtil.date(durationDTO.getOpenTime());
                DateTime endOffset = DateUtil.date(durationDTO.getCloseTime());
                if (startOffset.compareTo(BusinessConstant.DATE_FLAG) < 0 && endOffset.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                    //如果时间超过当天,舍弃
                    this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                            startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                            BusinessConstant.END_OF_DAY_TIME,
                            BusinessConstant.ONE, startDate
                    );
                    this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                            BusinessConstant.START_OF_DAY_TIME,
                            endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                            BusinessConstant.ONE, startDate
                    );
                    tundraStrategyList.add(
                            SpaceOptimizeDurationDTO.builder()
                                    .wellheadId(wellheadId)
                                    .openTime(startOffset)
                                    .closeTime(BusinessConstant.DATE_FLAG)
                                    .build()
                    );
                    tundraStrategyList.add(
                            SpaceOptimizeDurationDTO.builder()
                                    .wellheadId(wellheadId)
                                    .openTime(BusinessConstant.DATE_FLAG_BEGIN)
                                    .closeTime(endOffset)
                                    .build()
                    );
                } else if (endOffset.compareTo(BusinessConstant.DATE_FLAG) == 0) {
                    this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                            startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                            BusinessConstant.END_OF_DAY_TIME,
                            BusinessConstant.ONE, startDate
                    );
                    tundraStrategyList.add(
                            SpaceOptimizeDurationDTO.builder()
                                    .wellheadId(wellheadId)
                                    .openTime(startOffset)
                                    .closeTime(BusinessConstant.DATE_FLAG)
                                    .build()
                    );
                } else {
                    //计算偏移
                    this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                            startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                            endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                            BusinessConstant.ONE, startDate
                    );
                    tundraStrategyList.add(
                            SpaceOptimizeDurationDTO.builder()
                                    .wellheadId(wellheadId)
                                    .openTime(startOffset)
                                    .closeTime(endOffset)
                                    .build()
                    );
                }
            }
        }
    }

    /**
     * 添加离网优化时间
     *
     * @param durationDTOList 持续时间dtolist
     * @param offGridDTOList  脱离网格dtolist
     * @param weightList      重量清单
     */
    private void addOffGridOptimizeDuration(List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> offGridDTOList, List<SpaceOptimizeWeight> weightList) {
        if (CollUtil.isNotEmpty(offGridDTOList)) {
            SpaceOptimizeDurationDTO durationDTO;
            int index = 0;
            DateTime beginTime = null;
            String beginTypeKey = null;
            SpaceOptimizeWeight weight;
            for (int i1 = 0; i1 < offGridDTOList.size(); i1++) {
                durationDTO = offGridDTOList.get(i1);
                DateTime startTime = DateUtil.parse(durationDTO.getOpenWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                DateTime endTime = DateUtil.parse(durationDTO.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                for (int i2 = index; i2 < weightList.size(); i2++) {
                    weight = weightList.get(i2);
                    if (DateUtil.isIn(weight.getTimestamp(), startTime, endTime)) {
                        if (CharSequenceUtil.isBlank(beginTypeKey)) {
                            beginTypeKey = weight.getGenerationTypeKey();
                            beginTime = startTime;
                        }
                        if (!CharSequenceUtil.equals(beginTypeKey, weight.getGenerationTypeKey())) {
                            this.createOptimizeDuration(durationDTOList, durationDTO.getPeriodId(), durationDTO.getRecordId(), durationDTO.getWellheadId(), beginTypeKey,
                                    beginTime.toString(BusinessConstant.MINUTES_FORMAT),
                                    weight.getTimestamp().toString(BusinessConstant.MINUTES_FORMAT),
                                    BusinessConstant.ONE, durationDTO.getOptimizeDate()
                            );
                            beginTime = weight.getTimestamp();
                            beginTypeKey = weight.getGenerationTypeKey();
                        }
                    }
                    //时间戳大于结束时间,跳出循环设置下次索引位置
                    if (DateUtil.compare(weight.getTimestamp(), endTime) > 0) {
                        this.createOptimizeDuration(durationDTOList, durationDTO.getPeriodId(), durationDTO.getRecordId(), durationDTO.getWellheadId(), beginTypeKey,
                                beginTime.toString(BusinessConstant.MINUTES_FORMAT),
                                endTime.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ONE, durationDTO.getOptimizeDate()
                        );
                        index = i2;
                        beginTypeKey = null;
                        break;
                    }
                }
            }
        }
    }

    /**
     * 获取间开井口列表
     *
     * @param context        上下文
     * @param institutionIds 机构ID
     * @return {@link List }<{@link SpaceInstitutionWellheadView }>
     */
    private List<SpaceInstitutionWellheadView> getSpaceWellheadList(XContext context, List<String> institutionIds) {
        //取间开的所有井口
        SpaceInstitutionWellheadViewMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
        return wellheadMapper.selectWellheadsByInstitutionIds(institutionIds);
    }

    /**
     * 设置参数
     *
     * @param powerGenerationList 发电清单
     * @param electricityCostList 电费清单
     * @param optimizeType        优化类型
     * @param wellheadList        井口清单
     * @return {@link Map }
     */
    private Map<String, Object> setParams(List<BigDecimal> powerGenerationList, List<BigDecimal> electricityCostList, Integer optimizeType, List<SpaceInstitutionWellheadView> wellheadList) {
        Map<String, Object> param = new HashMap<>(8);
        param.put("secretKey", "fZ1kC1aV8bO7rH3bL8oE5zI0mY3vW1sG");
        param.put("pvPower", JSONUtil.toJsonStr(powerGenerationList));
        param.put("price", JSONUtil.toJsonStr(electricityCostList));
        param.put("pvPrice", BusinessConstant.BIG_DECIMAL_0_3587);
        param.put("type", optimizeType);
        List<Map<Object, Object>> wellParams = new ArrayList<>(wellheadList.size());
        for (SpaceInstitutionWellheadView view : wellheadList) {
            wellParams.add(MapUtil.builder()
                    .put("id", view.getWellheadId())
                    .put("name", view.getWellNumber())
                    .put("power", view.getServiceRating())
                    .put("totalOn", view.getRunDuration())
                    .put("maxSegment", view.getMaxOpen())
                    .put("minSegment", view.getMinOpen())
                    .put("maxSpace", view.getMaxClose())
                    .put("minSpace", view.getMinClose())
                    .build());
        }
        param.put("stations", JSONUtil.toJsonStr(wellParams));
        return param;
    }

    /**
     * 电费列表
     *
     * @param rangeToList  要列出范围
     * @param strategyList 战略列表
     * @return {@link List }<{@link BigDecimal }>
     */
    private List<BigDecimal> electricityCostList(List<DateTime> rangeToList, List<GetBasePriceStrategyDetailOutput> strategyList) {
        List<BigDecimal> electricityCostList = new ArrayList<>();
        Date closeTime;
        for (DateTime dateTime : rangeToList) {
            for (GetBasePriceStrategyDetailOutput strategy : strategyList) {
                closeTime = ObjectUtil.isEmpty(strategy.getCloseTime()) && CharSequenceUtil.equals(BusinessConstant.END_OF_DAY_TIME, strategy.getEndTime()) ? BusinessConstant.DATE_FLAG : strategy.getCloseTime();
                if (DateUtil.compare(dateTime, strategy.getOpenTime()) >= 0 && DateUtil.compare(dateTime, closeTime) < 0) {
                    electricityCostList.add(strategy.getElectrovalence());
                }
            }
        }
        return electricityCostList;
    }

    /**
     * 发电列表
     *
     * @param rangeToList 要列出范围
     * @param powerList   电源列表
     * @return {@link List }<{@link BigDecimal }>
     */
    private List<BigDecimal> powerGenerationList(List<DateTime> rangeToList, List<DynamicQueryPlantPredictedPowerOutput> powerList) {
        List<BigDecimal> electricityCostList = new ArrayList<>();
        DateTime dateTime;
        BigDecimal result = null;
        for (int i = 1; i < rangeToList.size(); i++) {
            dateTime = rangeToList.get(i);
            for (DynamicQueryPlantPredictedPowerOutput power : powerList) {
                if (DateUtil.compare(dateTime, power.getCreateTime()) == 0) {
                    result = power.getPower().setScale(4, RoundingMode.HALF_UP);
                }
            }
            electricityCostList.add(ObjectUtil.defaultIfNull(result, BigDecimal.ZERO));
        }
        return electricityCostList;
    }
}